# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
account_data.py

Provide data and functions for handle additional account and socket informations
'''

import mudsys, mud, auxiliary, storage, time

################################################################################
# account data type definition
################################################################################

class AccountData:
    '''Data for account.'''

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.encoding = "bez zmian"
        self.lastvisited = time.time()

        if set != None:
            self.encoding = set.readString("encoding")
            self.lastvisited = set.readDouble("lastvisited")

    def copyTo(self, to):
        to.encoding = self.encoding
        to.lastvisited = self.lastvisited

    def copy(self):
        newData = AccountData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString("encoding", self.encoding)
        set.storeDouble("lastvisited", self.lastvisited)
        return set

################################################################################
# socket data type definition
################################################################################

class SocketData:
    '''Data for socket.'''

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.encoding = "brak"
        self.mxp = False

        if set != None:
            self.encoding = set.readString("encoding")
            self.mxp = set.readBool("mxp")

    def copyTo(self, to):
        to.encoding = self.encoding
        to.mxp = self.mxp

    def copy(self):
        newData = SocketData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString("encoding", self.encoding)
        set.storeBool("mxp", self.mxp)
        return set

################################################################################
# initialization
################################################################################
auxiliary.install("misc", SocketData, "socket")
auxiliary.install("data", AccountData, "account")
