# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
alignment.py

Provide data and functions for alignment for characters.
'''

import mudsys, mud, auxiliary, storage, olc, char, hooks

################################################################################
# alignment data type definition
################################################################################

class AlignmentData:
    '''Alignment character data.'''

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.value = 0
        self.desc = "Neutralny"

        if set != None:
            if len(set.readString('desc')) > 0:
                self.desc = set.readString('desc')
                self.value = set.readInt('value')

    def copyTo(self, to):
        to.value = self.value
        to.desc = self.desc

    def copy(self):
        newData = AlignmentData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeInt('value', self.value)
        set.storeString('desc', self.desc)
        return set

################################################################################
# functions
################################################################################

def count_alignment(ch):
    '''Set description of alignment for character and check if player can still believe in selected god.'''
    if ch.aux('alignment').value > 100:
        ch.aux('alignment').value = 100
    if ch.aux('alignment').value < -100:
        ch.aux('alignment').value = -100
    if ch.aux('alignment').value > 50:
        desc = 'Dobry'
    elif ch.aux('alignment').value > 0 and ch.aux('alignment').value < 51:
        desc = 'Neutralny Dobry'
    elif ch.aux('alignment').value == 0:
        desc = 'Neutralny'
    elif ch.aux('alignment').value < 0 and ch.aux('alignment').value > -51:
        desc = 'Neutralny Zły'
    else:
        desc = 'Zły'
    if desc != ch.aux('alignment').desc:
        ch.send("Twój charakter zmienia się.")
        ch.aux('alignment').desc = desc

    #no point in continue
    if ch.aux('faith').deity == 'Ateista':
        return
    if ch.aux('alignment').value < ch.aux('faith').minalign or ch.aux('alignment').value > ch.aux('faith').maxalign:
        ch.aux('faith').deity = 'Ateista'
        ch.aux('faith').value = 0
        ch.aux('faith').desc = ''
        ch.send("W związku z twoimi ostatnimi poczynaniami, twoje bóstwo, nie chce mieć więcej z tobą do czynienia.")

################################################################################
# Alignment OLC
################################################################################

def alignolc_chooser(sock, data):
    '''Show menu.'''
    try:
        sock.send("Nowa wartość charakteru: ")
        return 1
    except:
        return olc.MENU_CHOICE_INVALID

def alignolc_parser(owner, arg):
    '''Set new alignment.'''
    try:
        arg = int(arg)
    except:
        return False
    owner.aux('alignment').value = arg
    count_alignment(owner)
    return True

def alignolc_to_proto(owner):
    '''Save alignment to file.'''
    buf = 'me.aux("alignment").value = %d\n' % owner.aux('alignment').value
    buf += 'me.aux("alignment").desc = "%s"\n' % owner.aux('alignment').desc
    return buf

def alignolc_extend_menu(sock, data):
    '''Show option in main menu'''
    sock.send('Charakter: %s (%d)' % (data.aux('alignment').desc, data.aux('alignment').value))

################################################################################
# game hooks
################################################################################

def align_hook(info):
    '''When player alignment value was changed, get new description for it.'''
    ch, = hooks.parse_info(info)
    if ch.is_pc:
        count_alignment(ch)

################################################################################
# initialization
################################################################################
auxiliary.install("alignment", AlignmentData, "character")

olc.extend("medit", "H", alignolc_extend_menu, alignolc_chooser, alignolc_parser, None,
           alignolc_to_proto)
olc.extend("pcedit", "H", alignolc_extend_menu, alignolc_chooser, alignolc_parser, None, None)

hooks.add("alignment", align_hook)

def __unload__():
    '''remove the hooks for alignment.'''
    hooks.remove('alignment', align_hook)
