# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria MUD.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
ban.py

handle ban via IP.
'''
import mudsys, storage, hooks, socket

################################################################################
# functions
################################################################################
banlist_file = '../lib/misc/banlist'
banlist = []

def load_banlist():
    '''Read banned IP from file.'''
    set = storage.StorageSet(banlist_file)
    for ban in set.readList("list").sets():
        banlist.append(ban.readString("val"))
    set.close()

def save_banlist():
    '''Saves all banned IP to file.'''
    set = storage.StorageSet()
    list = storage.StorageList()
    set.storeList("list", list)
    for val in banlist:
        one_set = storage.StorageSet()
        one_set.storeString("val", val)
        list.add(one_set)
    set.write(banlist_file)
    set.close()

################################################################################
# game commands
################################################################################

def cmd_banip (ch, cmd, arg):
    '''Usage: banip <IP>

    Ban/unban selected IP.'''
    if arg in banlist:
        banlist.remove(arg)
        ch.send('Odbanowałeś IP ' + arg)
    else:
        banlist.append(arg)
        save_banlist()
        for sock in socket.socket_list():
            if sock.hostname == arg:
                sock.close()
        ch.send('Zbanowałeś IP ' + arg)

################################################################################
# hooks
################################################################################

def ban_hook(info):
    '''When a socket connects, check if it is not banned.'''
    sock, = hooks.parse_info(info)
    if sock.hostname in banlist:
        sock.close()

################################################################################
# initialization
################################################################################
load_banlist()
mudsys.add_cmd("banip", None, cmd_banip, "admin", False)
hooks.add("receive_connection", ban_hook)

def __unload__():
    '''removes hook from game.'''
    hooks.remove("reveive_connection", ban_hook)
