# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
books.py

Books item type.
'''
import olc, mudsys, auxiliary, mudsock, storage, mud, hooks

################################################################################
# item type definition
################################################################################
booktext = ""

class BookData:
    '''Data for books - title and body.'''

    __item_type__ = "book"

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.title = ""
        self.body = ""

        if set != None:
            self.title = set.readString('title')
            self.body = set.readString('body')

    def copyTo(self, to):
        to.title = self.title
        to.body = self.body

    def copy(self):
        newData = BookData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString('title', self.title)
        set.storeString('body', self.body)
        return set

################################################################################
# functions
################################################################################

def finish_book_editor(sock, editor_output):
    '''this function is passed to edit_text, and called after a socket exits the
       text editor.'''
    if sock != None:
        global booktext
        booktext = editor_output

################################################################################
# Books OLC
################################################################################

def bookolc_menu(sock, data):
    '''display our menu interface.'''
    global booktext
    if len(booktext) == 0:
        booktext = data.body
    buf = ["{g1) Tytuł : {c" + data.title,
           "{g2) Treść : {c" + booktext]
    sock.send('\r\n'.join(buf))

def bookolc_chooser(sock, data, option):
    '''figure out what field we want to edit, and display a prompt.'''
    if option == '1':
        sock.send("Nowy tytuł książki: ")
        return 1
    elif option == '2':
        global booktext
        sock.edit_text(booktext, finish_book_editor)
        return olc.MENU_NOCHOICE

def bookolc_parser(sock, item, choice, arg):
    '''Set new values for book settings.'''
    try:
        option = int(choice)
    except:
        return False
    if option == 1:
        item.title = arg
    else:
        return False
    return True

def bookolc_to_proto(data):
    '''return a script that will generate relevant info to create this item.'''
    global booktext
    if len(booktext) == 0:
        booktext = data.body
    buf = 'me.get_type_data("book").title = "' + data.title + '"\n'
    buf += 'me.get_type_data("book").body = "' + booktext + '"\n'
    return buf

################################################################################
# game hooks
################################################################################
def book_desc_info(info):
    '''appends books informations about an object to the description buffer
       when person looks at it.'''
    obj, ch = hooks.parse_info(info)

    #no point in continuing
    if not obj.istype("book"):
        return

    if len(obj.get_type_data("book").title) == 0:
        desc = " Nie posiada tytułu"
    else:
        desc = " Jej tytuł to: " + obj.get_type_data("book").title

    if len(obj.get_type_data("book").body) == 0:
        desc += " i nie zawiera jakiejkolwiek treści."
    else:
        desc += " i zawiera jakiś tekst."

    ch.look_buf += desc

################################################################################
# game commands
################################################################################
def cmd_book(ch, cmd, arg):
    '''Allow to read, write books.'''
    global booktext
    #if no argument, show what books player have
    if len(arg) == 0:
        books = []
        for book in ch.inv:
            if not book.istype('book'):
                continue
            if len(book.get_type_data("book").title) == 0:
                title = 'Bez tytułu'
            else:
                title = book.get_type_data("book").title
            if len(book.get_type_data("book").body) == 0:
                body = 'Czysta'
            else:
                body = 'Zawiera tekst'
            books.append([book.name, title, body])
        if len(books) == 0:
            ch.send("Nie posiadasz jakiejkolwiek książki.")
        else:
            buf = "Posiadane książki:\r\n"
            for book in books:
                buf += book[0] + " " + book[1] + " " + book[2] + "\r\n"
            ch.send(buf)
        return
    options = arg.split()
    #check did player have that item
    try:
        book, = mud.parse_args(ch, True, cmd, options[0], "obj.inv")
    except:
        return
    #check if this is a book
    if not book.istype('book'):
        ch.send("To nie jest książka.")
        return
    optlist = ['czytaj', 'pisz', 'tytul', 'zapisz']
    if len(options) < 2:
        ch.send("Dostępne opcje to: czytaj, pisz, tytul, zapisz")
        return
    if not options[1] in optlist:
        ch.send("Dostępne opcje to: czytaj, pisz, tytul, zapisz")
        return
    #read book
    if options[1] == 'czytaj':
        if len(book.get_type_data("book").body) == 0:
            ch.send("Ta książka nie zawiera tekstu.")
            return
        if len(book.get_type_data("book").title) > 0:
            ch.send("Tytuł: " + book.get_type_data("book").title)
        ch.page(book.get_type_data("book").body)
    #set book title
    elif options[1] == 'tytul':
        if len(options) < 3:
            ch.send("Podaj nowy tytuł książki.")
            return
        book.get_type_data("book").title = " ".join(options[2:])
        ch.send("Tytuł książki został zmieniony.")
    #write a book
    elif options[1] == 'pisz':
        ch.sock.edit_text(booktext, finish_book_editor)
    #save a book
    elif options[1] == 'zapisz':
        book.get_type_data("book").body = booktext
        ch.send("Książka została zapisana.")

################################################################################
# initialization
################################################################################

#initialize our item data, and its OLC
mudsys.item_add_type(BookData.__item_type__, BookData)
olc.item_add_olc(BookData.__item_type__,
                 bookolc_menu, bookolc_chooser, bookolc_parser,
                 None, bookolc_to_proto)

#display hooks for books
hooks.add("append_obj_desc", book_desc_info)

#add game commands
mudsys.add_cmd("ksiazka", None, cmd_book, "player", False)

def __unload__():
    '''remove all hooks.'''
    hooks.remove('append_obj_desc', book_desc_info)
