# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
char_gen.py

The basic character generation module. Allows accounts to create new characters
with basic selection for name, sex, and race.
'''
import mudsys, mud, socket, char, hooks, random



def check_char_name(arg):
    '''checks to make sure the character name is valid. Names are valid if they
       are greater than 2 characters, less than 13, and comprise only alpha
       characters.'''
    return len(arg) >= 3 and len(arg) <= 12 and arg.isalpha()

def cg_name_handler(sock, arg):
    if not check_char_name(arg):
        sock.send("{cNieprawidłowe imię postaci, proszę wybrać inne.{n\r\n")
    elif mudsys.player_exists(arg):
        sock.send("{cPostać o tym imieniu już istnieje.{n\r\n")
    elif mudsys.player_creating(arg):
        sock.send("{cPostać o tym imieniu już jest tworzona.{n\r\n")
    elif arg.lower().startswith("gość"):
        sock.send("{cImię postaci nie może zaczynać się od słowa 'gość'.{n\r\n")
    else:
        name = arg[0].upper() + arg[1:]
        ch = mudsys.create_player(name)

        if ch == None:
            sock.send("{cNieprawidłowe imię postaci, proszę wybrać inne.{n\r\n")
        else:
            mudsys.attach_char_socket(ch, sock)
            ch.rdesc = ch.name + " jest tutaj."
            sock.pop_ih()

def cg_sex_handler(sock, arg):
    try:
        result = {
            'M' : 'mężczyzna',
            'K' : 'kobieta',
            }[arg[0].upper()]
        sock.ch.sex = result
        sock.pop_ih()
    except:
        sock.send("{cNieprawidłowa płeć, spróbuj ponownie.{n\r\n")

def cg_race_handler(sock, arg):
    races = mud.list_races(True).split(', ')
    arg = int(arg)
    if arg < 0 or arg > len(races):
        sock.send("{cNieprawidłowa rasa, spróbuj ponownie.{n\r\n")
    else:
        sock.ch.race = races[arg]
        race = sock.ch.race
        oldchars = ['ą', 'ć', 'ę', 'ó', 'ś', 'ł', 'ż', 'ź', 'ń']
        newchars = ['a', 'c', 'e', 'o', 's', 'l', 'z', 'z', 'n']
        i = 0
        for char in oldchars:
            race = race.replace(char, newchars[i])
            i += 1
        sock.send(mudsys.get_help(race)[1])
        if sock.ch.sex == "mężczyzna":
            weight = random.randint(65, 80)
        else:
            weight = random.randint(50, 65)
        values = sock.ch.aux('stats').values
        secondvalues = sock.ch.aux('stats').secondvalues
        if sock.ch.race == "elf":
            values[0] = 4
            values[1] = 6
            weight -= 5
        elif sock.ch.race == "goblin":
            values[0] = 4
            values[3] = 6
            weight -= 15
        elif sock.ch.race == "jaszczuroczłek":
            values[1] = 4
            values[3] = 6
            weight += 10
        elif sock.ch.race == "gnom":
            values[0] = 4
            values[1] = 6
            weight -= 20
        elif sock.ch.race == "niziołek":
            values[0] = 3
            values[1] = 6
            values[3] = 6
            weight -= 25
        elif sock.ch.race == "faerie":
            values[0] = 2
            values[1] = 7
            weight -= 55
        elif sock.ch.race == "kotołak":
            values[1] = 6
            values[3] = 4
            weight -= 10
        elif sock.ch.race == "krasnolud":
            values[0] = 6
            values[1] = 4
        #count rest of player stats
        values[4] = values[0] + values[3]
        secondvalues[4] = values[4]
        secondvalues[5] = values[4] * 10
        secondvalues[6] = 20 + values[3]
        secondvalues[7] = 20 + values[3]
        sock.ch.aux('stats').values = values
        sock.ch.aux('stats').secondvalues = secondvalues
        sock.ch.weight = weight
        sock.ch.worth = {"miedziaki" : 1000}
        sock.replace_ih(cg_confirm_handler, cg_confirm_prompt)

def cg_confirm_handler(sock, arg):
    '''Confirm selected race handler.'''
    if arg.lower() == 't':
        sock.replace_ih(cg_finish_handler, cg_finish_prompt)
    elif arg.lower() == 'n':
        sock.replace_ih(cg_race_handler, cg_race_prompt)

def cg_finish_handler(sock, arg):

    # log that the character created
    mud.log_string("New player: " + sock.ch.name + " has entered the game.")
    
    # register and save him to disk and to an account
    mudsys.do_register(sock.ch)

    # make him exist in the game for functions to look him up
    mudsys.try_enter_game(sock.ch)

    # run the init_player hook
    hooks.run("init_player", hooks.build_info("ch", (sock.ch,)))
    
    # attach him to his account and save the account
    sock.account.add_char(sock.ch)
    mudsys.do_save(sock.account)
    mudsys.do_save(sock.ch)

    # clear their screen
    sock.ch.act("wyczysc")
    
    # send them the motd
    sock.ch.page(mud.get_motd())

    # make him look at the room
    sock.ch.act("patrz")

    # run our enter hook
    hooks.run("enter", hooks.build_info("ch rm", (sock.ch, sock.ch.room)))

    sock.replace_ih(mudsys.handle_cmd_input, mudsys.show_prompt, "playing")

def cg_name_prompt(sock):
    sock.send_raw("Jakie będzie imię twojej postaci? ")

def cg_sex_prompt(sock):
    sock.send_raw("Jaka będzie jej płeć (M/K)? ")

def cg_race_prompt(sock):
    sock.send("Dostępne rasy: ")
    i = 0
    buf = []
    for race in mud.list_races(True).split(', '):
        buf.append("[{c%d{n] %s" % (i, race))
        i += 1
    sock.send('\r\n'.join(buf))
    sock.send_raw("\r\nWybierz rasę: ")

def cg_finish_prompt(sock):
    sock.send_raw("{c*** Naciśnij enter aby zakończyć tworzenie postaci:{n ")

def cg_confirm_prompt(sock):
    '''Confirm selected race prompt.'''
    sock.send("Czy chcesz wybrać tę rasę (T/N)? ")



################################################################################
# character generation hooks
################################################################################
def char_gen_hook(info):
    '''Put a socket into the character generation menu when character generation
       hooks are called.
    '''
    sock, = hooks.parse_info(info)
    sock.push_ih(cg_race_handler, cg_race_prompt)
    sock.push_ih(cg_sex_handler, cg_sex_prompt)
    sock.push_ih(cg_name_handler, cg_name_prompt)

def guest_gen_hook(info):
    sock, = hooks.parse_info(info)
    sock.push_ih(mudsys.handle_cmd_input, mudsys.show_prompt, "playing")

    # make the guest character
    ch = mudsys.create_player("Gość")

    # oops...
    if ch == None:
        sock.send("Przykro mi, wystąpiły problem podczas tworzenia konta gościa.")
        sock.close()

    mudsys.attach_char_socket(ch, sock)
    ch.rdesc = "Gość jest tutaj, zwiedza świat."
    ch.name  = ch.name + str(ch.uid)
    ch.race  = "człowiek"

    # log that the character created
    mud.log_string("Guest character created (id %d)." % ch.uid)

    # make him exist in the game for functions to look him up
    mudsys.try_enter_game(ch)

    # run the init_player hook
    hooks.run("init_player", hooks.build_info("ch", (ch,)))

    # clear our screen
    ch.act("wyczysc")

    # send them the motd
    ch.page(mud.get_motd())
    
    # make him look at the room
    ch.act("patrz")

    # run our enter hook
    hooks.run("enter", hooks.build_info("ch rm", (ch, ch.room)))



################################################################################
# loading and unloading the module
################################################################################
hooks.add("create_character", char_gen_hook)
hooks.add("create_guest",     guest_gen_hook)

def __unload__():
    '''removes the hooks for character generation'''
    hooks.remove("create_character", char_gen_hook)
    hooks.remove("create_guest",     guest_gen_hook)

