# -*- coding: utf-8 -*-
'''
cmd_inform.py

Contains various commands that are informative in nature. For instance, look,
equipment, inventory, etc...
'''
import mud, mudsys, inform, utils



################################################################################
# utility functions
################################################################################
def cmd_inventory(ch, cmd, arg):
    '''Lists all of the items currently carried in your inventory.'''
    if len(ch.inv) == 0:
        ch.send("Nic nie niesiesz.")
    else:
        ch.send("Niesiesz:")
        visible = utils.find_all_objs(ch, ch.inv, "", None, True)
        utils.show_list(ch, visible, lambda(x): x.name, lambda(x): x.mname)
        weight = inform.count_weight(ch)
        buf = []
        buf.append("Waga: %g kg (razem z ekwipunkiem: %g kg)" % (weight[2], weight[0]))
        buf.append("Twój maksymalny udźwig to %g kg" % weight[3])
        ch.send("\r\n".join(buf))

def cmd_equipment(ch, cmd, arg):
    '''Displays all of the equipment you are currently wearing.'''
    ch.send("Nosisz na sobie:")
    inform.show_equipment(ch, ch)
    weight = inform.count_weight(ch)
    if weight[1] > 0:
        buf = []
        buf.append("Waga: %g kg (razem z inwentarzem: %g kg)" % (weight[1], weight[0]))
        buf.append("Twój maksymalny udźwig to %g kg" % weight[3])
        ch.send("\r\n".join(buf))

def cmd_who(ch, cmd, arg):
    '''List all of the players currently online.'''
    ch.page(inform.build_who(ch))
    
def cmd_worth(ch, cmd, arg):
    '''Display amount of money of player.'''
    money = str(ch.worth).split()
    if len(money) == 1:
        ch.send("Nie masz pieniędzy.")
        return
    if money[1] == "md":
        money[1] = "miedziaków"
    elif money[1] == "sr":
        money[1] = "srebrników"
    elif money[1] == "zł":
        money[1] = "złociszy"
    ch.send("Pieniądze: " + money[0] + " " + money[1])
    bank = ch.getvar('bank')
    if bank > 0:
        ch.send("W banku: " + str(bank) + " miedziaków")



################################################################################
# add our commands
################################################################################
mudsys.add_cmd("inwentarz", "i",   cmd_inventory, "player", False)
mudsys.add_cmd("ekwipunek", "ek",  cmd_equipment, "player", False)
mudsys.add_cmd("noszone",      None,  cmd_equipment, "player", False)
mudsys.add_cmd("kto",       None,  cmd_who,       "player", False)
mudsys.add_cmd("sakiewka", None, cmd_worth, "player", False)
