# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria MUD.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
cmd_manip.py

a set of commands that NakedMud(tm) comes with that allows characters to
manipulate various things. These commands are mostly directed towards
manipulating objects (e.g. get, put, drop, etc...) but can also affect other
things like exits (e.g. open, close)
'''
import mudsys, mud, inform, utils, movement, hooks, char, random
import obj as mudobj



def do_give(ch, recv, obj):
    '''does the handling of the give command'''
    weight = inform.count_weight(recv)
    newweight = weight[0] + obj.weight
    if newweight > weight[3]:
        ch.send(recv.name + " nie uniesie " + ch.see_as(obj) + "$g.")
        return
    mud.message(ch, recv, obj, None, True, "to_room",
                "$n dał(a) $N $o.")
    mud.message(ch, recv, obj, None, True, "to_vict",
                "$n dał(a) tobie $o.")
    mud.message(ch, recv, obj, None, True, "to_char",
                "Dajesz $N $o.")
    obj.carrier = recv

    # run our give hook
    hooks.run("give", hooks.build_info("ch ch obj", (ch, recv, obj)))

def cmd_give(ch, cmd, arg):
    '''Usage: give <object> [to] <person>

       Attempts to transfer an object from your inventory to the specified
       person. You can give multiple objects at a time by using the all.
       prefix. For example:

       > daj all.cookie george

       Would give all of the cookies in your inventory to George. If you only
       want to give him the raisin cookie, which also happens to be the third
       cookie in your inventory, you can use a numeric prefix instead of
       the all prefix. For example:

       > daj 3.cookie george'''
    try:
        to_give, multi, recv = mud.parse_args(ch, True, cmd, arg,
                                              "obj.inv.multiple " +
                                              "ch.room.noself")
    except: return

    if ch.sock.account == recv.sock.account:
        ch.send("Nie możesz dać przedmiotu postaci z tego samego konta.")
        return

    if multi == False:
        do_give(ch, recv, to_give)
    else:
        for obj in to_give:
            do_give(ch, recv, obj)
    
def do_get(ch, obj, cont):
    '''transfers an item from the ground to the character'''
    if utils.is_keyword(obj.bits, "notake"):
        ch.send("Nie możesz wziąć " + ch.see_as(obj) + "$a.")
        return
    if obj.hasvar('owner'):
        if obj.getvar('owner') != ch.name:
            mud.message(ch, None, obj, None, True, "to_char", "Nie możesz wziąć " + ch.see_as(obj) + "$$a.")
            return
    weight = inform.count_weight(ch)
    newweight = weight[0] + obj.weight
    if newweight > weight[3]:
        ch.send("Nie uniesiesz " + ch.see_as(obj) + "$g.")
        return
    if cont != None:
        obj.carrier = ch
        mud.message(ch, None, obj, cont, True, "to_char", "Bierzesz $o z $O$$g.")
        mud.message(ch, None, obj, cont, True, "to_room", "$n bierze $o z $O$$g.")
    else:
        obj.carrier = ch
        mud.message(ch, None, obj, None, True, "to_char", "Bierzesz $o.")
        mud.message(ch, None, obj, None, True, "to_room", "$n bierze $o.")

    # run get hooks
    hooks.run("get", hooks.build_info("ch obj", (ch, obj)))

def try_get_from(ch, cont, arg):
    '''tries to get one item from inside another'''
    if not cont.istype("container"):
        mud.message(ch, None, cont, None, True, "to_char",
                    "$o nie jest pojemnikiem.")
        return
    elif cont.container_is_closed:
        mud.message(ch, None, cont, None, True, "to_char", "$o jest zamknięty.")
        return
    elif cont.hasvar('owner'):
        if cont.getvar('owner') != ch.name:
            mud.message(ch, None, cont, None, True, "to_char", "Nie możesz brać rzeczy z $o$$g.")
            return
    # find our count and name
    num, name = utils.get_count(arg)

    # multi or single?
    if num == "all":
        list = utils.find_all_objs(ch, cont.objs, name)
        for obj in list:
            do_get(ch, obj, cont)
    else:
        # obj = find_obj(ch, cont.objs, num, name)
        obj = mudobj.find_obj(arg, cont, ch)
        if obj != None:
            do_get(ch, obj, cont)
        else:
            mud.message(ch, None, cont, None, True, "to_char",
                        "Nie możesz znaleźć $o$$g.")

def cmd_get(ch, cmd, arg):
    '''Usage: get [the] <item> [[from] <other item>]

       Attempts to move an object from the room to your inventory. If an
       addition argument is supplied, the command assumes it is a container and
       instead tries to move an object from the container to your inventory.'''
    try:
        arg,cont = mud.parse_args(ch, True, cmd, arg,
                                  "word(object) | [z] obj.room.inv.eq")
    except: return
    
    # are we doing get, or get-from?
    if cont != None:
        try_get_from(ch, cont, arg)
    else:
        # try to find the object in the room
        try:
            found,multi= mud.parse_args(ch, True, cmd, arg, "obj.room.multiple")
        except: return

        # pick up all the items we want
        if multi == False:
            do_get(ch, found, None)
        else:
            for obj in found:
                do_get(ch, obj, None)

def do_drop(ch, obj):
    '''handles object dropping'''
    #check if player is in shop
        
    obj.room = ch.room

    mud.message(ch, None, obj, None, True, "to_char", "Upuszczasz $o.")
    mud.message(ch, None, obj, None, True, "to_room", "$n upuszcza $o.")

    # run our drop hook
    hooks.run("drop", hooks.build_info("ch obj", (ch, obj)))

def cmd_drop(ch, cmd, arg):
    '''Usage: drop <item>

       Attempts to move an object from your inventory to the ground.'''
    try:
        found,multi=mud.parse_args(ch, True, cmd, arg, "obj.inv.multiple")
    except: return

    # are we dropping a list of things, or just one?
    if multi == False:
        do_drop(ch, found)
    else:
        for obj in found:
            do_drop(ch, obj)

def do_sacrifice(ch, obj):
    '''handles object sacrifice.'''
    if utils.is_keyword(obj.bits, "nosac"):
        ch.send("Nie możesz poświęcić " + obj.name + "$g.")
        return
    if obj.hasvar('owner'):
        if obj.getvar('owner') != ch.name:
            ch.send("Nie możesz poświęcić " + obj.name + "$g.")
            return
    mud.extract(obj)
    ch.worth += {'miedziaki' : 1}
    if ch.aux('faith').deity == 'Brak':
        mud.message(ch, None, obj, None, True, "to_char", "Poświęcasz $o. Bogowie dają tobie w zamian 1 miedziaka.")
        mud.message(ch, None, obj, None, True, "to_room", "$n poświęca bogom $o.")
    else:
        mud.message(ch, None, obj, None, True, "to_char", "Poświęcasz $o. " + ch.aux('faith').deity + " łaskawiej spogląda na ciebie i daje tobie w zamian 1 miedziaka.")
        mud.message(ch, None, obj, None, True, "to_room", "$n poświęca $o swojemu bóstwu.")
        ch.aux('faith').value += 1
        hooks.run("faith", hooks.build_info("ch", (ch,)))

def cmd_sacrifice(ch, cmd, arg):
    '''Usage: poswiec <przedmiot>

       Sacrifice object to gods (remove it from game).'''
    try:
        found, type, multi = mud.parse_args(ch, True, cmd, arg, "{ obj.inv.multiple obj.room.multiple }")
    except:
        return
    
    # are we sacrifice a list of things, or just one?
    if multi == False:
        do_sacrifice(ch, found)
    else:
        for obj in found:
            do_sacrifice(ch, obj)

def do_remove(ch, obj):
    '''handles equipment removing'''
    # try to put it to our inventory
    obj.carrier = ch

    # make sure it succeeded
    if obj.carrier != ch:
        ch.send("Nie możesz zdjąć " + ch.see_as(obj) + "$g.")
    else:
        mud.message(ch, None, obj, None, True, "to_char", "Zdejmujesz $o$$a.")
        mud.message(ch, None, obj, None, True, "to_room", "$n zdejmuje $o$$a.")

        # run our hooks
        hooks.run("remove", hooks.build_info("ch obj", (ch, obj)))

def cmd_remove(ch, cmd, arg):
    '''Usage: remove <item | all>

       Attempts to remove an item you have equipped. If you would like to
       remove everything you are wearing, you may instead specify \'all\'
       instead of a specific item. If you would like to remove all of a certain
       type of object (for instance, rings) you can use an all. prefix. For
       example:

       > remove all.ring

       This command will remove everything you are wearing with the \'ring\'
       keyword. If you would instead like to remove, say, the second thing you
       are wearing with the \'ring\' keyword, you can supply a numeric prefix.
       For example:

       > remove 2.ring'''
    try:
        found,multi= mud.parse_args(ch, True, cmd, arg, "obj.eq.multiple")
    except: return

    # are we removing one thing, or multiple things?
    if multi == False:
        do_remove(ch, found)
    else:
        for obj in found:
            do_remove(ch, obj)

def do_wear(ch, obj, where):
    '''handles object wearing'''
    if not obj.istype("worn"):
        ch.send("Nie możesz założyć " + ch.see_as(obj) + "$g.")
        
    elif ch.equip(obj, where):
        mud.message(ch, None, obj, None, True, "to_char", "Zakładasz $o$$a.")
        mud.message(ch, None, obj, None, True, "to_room", "$n zakłada $o$$a.")

        # run our wear hook
        hooks.run("wear", hooks.build_info("ch obj", (ch, obj)))

def cmd_wear(ch, cmd, arg):
    '''Usage: wear <item> [where]

       Attempts to equip an item from your inventory. If you would like to
       equip it to a non-default location, you can supply where on your body
       you would like to wear it. For example, if you would like to equip a
       torch, but in your offhand instead of your mainhand:

       > wear torch offhand

       If an item covers multiple locations on your body, you can specify where
       all you would like to equip the item as a comma-separated list:

       > wear gloves left hand, right hand'''
    try:
        found, multi, where = mud.parse_args(ch, True, cmd, arg,
                                             "obj.inv.multiple | [na] string(bodyparts)")
    except: return

    # Are the things we're looking for not body positions? Try to catch this!
    # it will happen when someone enters multiple arguments for the name without
    # supplying ' and ' around it. The mud will misinterpret it as an item
    if not multi and where != None and not "," in where:
        # reparse what we want!
        if not where in ch.bodyparts:
            where = None
            try:
                found, = mud.parse_args(ch,True,cmd,"'"+arg+"'","obj.inv")
            except: return

    # are we wearing one thing, or multiple things?
    if multi == False:
        do_wear(ch, found, where)
    else:
        for obj in found:
            do_wear(ch, obj, where)

def do_put(ch, obj, cont):
    '''handles the putting of objects'''
    if obj == cont:
        ch.send("Nie możesz włożyć " + ch.see_as(obj) + "$g wewnątrz siebie.")
    # make sure we have enough room 
    elif obj.weight > cont.container_capacity - cont.weight + cont.weight_raw:
        ch.send("Nie ma miejsca w " + ch.see_as(cont) +
                "$l na " + ch.see_as(obj) + ".")
    elif cont.hasvar('owner'):
        if cont.getvar('owner') != ch.name:
            ch.send("Nie możesz wkładać rzeczy do " + cont.name)
    # do the move
    else:
        obj.container = cont
        mud.message(ch, None, obj, cont, True, "to_char", "Wkładasz $o do $O.")
        mud.message(ch, None, obj, cont, True, "to_room", "$n wkłada $o do $O.")

def cmd_put(ch, cmd, arg):
    '''Usage: put [the] <item> [in the] <container>

       Attempts to move an object from your inventory into a specified
       container. The container must be in the room, in your inventory, or
       worn.'''
    try:
        found, multi, cont = mud.parse_args(ch, True, cmd, arg,
                                            "obj.inv.multiple " +
                                            "[w] obj.room.inv")
    except: return

    # make sure we have a container
    if not cont.istype("container"):
        ch.send(ch.see_as(cont) + " nie jest pojemnikiem.")
    elif cont.container_is_closed:
        ch.send(ch.see_as(cont) + " jest zamknięta.")
    # do we have one or multiple items?
    elif multi == False:
        do_put(ch, found, cont)
    else:
        for obj in found:
            do_put(ch, obj, cont)

def try_manip_other_exit(room, ex, closed, locked):
    '''used by open, close, lock, and unlock. When an exit is manipulated on one
       side, it is the case that we\'ll want to do an identical manipulation on
       the other side. That\'s what we do here. Note: Can only do close OR lock
       with one call to this function. Cannot handle both at the same time!'''
    opp_dir = None
    if ex.dest == None:
        return

    # see if we can figure out the opposite direction
    if ex.opposite != '':
        opp_dir = ex.opposite
    else:
        # figure out the direction of the exit, and its opposite
        dirnum = movement.dir_index(room.exdir(ex))
        if dirnum != -1:
            opp_dir = movement.dir_name[movement.dir_opp[dirnum]]

    # do we have an opposite direction to manipulate?
    if opp_dir == None:
        return

    # do we have an opposite exit to manipulate?
    opp_ex = ex.dest.exit(opp_dir)
    if opp_ex != None:
        # figure out our name
        name = "an exit"
        if opp_ex.name != '':
            name = opp_ex.name

        # try to manipulate the exit
        if closed and not opp_ex.is_closed:
            opp_ex.close()
            ex.dest.send(name + " zostały zamknięte z drugiej strony.")
        elif locked and opp_ex.is_closed and not opp_ex.is_locked:
            opp_ex.lock()
            ex.dest.send(name + " zostały zablokowane z drugiej strony.")
        elif not closed and opp_ex.is_closed and not opp_ex.is_locked:
            opp_ex.open()
            ex.dest.send(name + " zostały otwarte z drugiej strony.")
        elif not locked and opp_ex.is_locked:
            opp_ex.unlock()
            ex.dest.send(name + " zostały odblokowane z drugiej strony.")
        else:
            return

def cmd_lock(ch, cmd, arg):
    '''Usage: lock <direction | door | container>

       Attempts to lock a specified door, direction, or container.'''
    try:
        found, type = mud.parse_args(ch, True, cmd, arg,
                                     "{obj.room.inv.eq exit }")
    except: return

    # what did we find?
    if type == "exit":
        ex = found
        name = ex.name
        if ex.name == "":
            name = "wyjście"
        
        if not ex.is_closed:
            ch.send(name + " musi być najpierw zamknięte.")
        elif ex.is_locked:
            ch.send(name + " jest już zablokowany.")
        elif ex.key == '':
            ch.send("Nie masz pojęcia jak zablokować " + name + ".")
        elif not utils.has_proto(ch, ex.key):
            ch.send("Nie możesz znaleźć klucza.")
        else:
            mud.message(ch, None, None, None, True, "to_char",
                        "Zamykasz " + name + ".")
            mud.message(ch, None, None, None, True, "to_room",
                        "$n zamyka " + name + ".")
            ex.lock()
            # hooks.run("room_change", hooks.build_info("rm", (ch.room, )))
            try_manip_other_exit(ch.room, ex, ex.is_closed, True)

    # type must be object
    else:
        obj = found
        if not obj.istype("container"):
            ch.send(ch.see_as(obj) + " to nie jest pojemnik.")
        elif not obj.container_is_closed:
            ch.send(ch.see_as(obj) + " nie jest zamknięty.")
        elif obj.container_is_locked:
            ch.send(ch.see_as(obj) + " już jest zablokowany.")
        elif obj.container_key == '':
            ch.send("Nie masz pojęcia jak zablokować " + ch.see_as(obj))
        elif not utils.has_proto(ch, obj.container_key):
            ch.send("Nie możesz znaleźć klucza.")
        else:
            mud.message(ch, None, obj, None, True, "to_char", "Zamykasz $o.")
            mud.message(ch, None, obj, None, True, "to_room", "$n zamyka $o.")
            obj.container_is_locked = True

def do_picklock(ch, objname, chance):
    '''Try open object with picklock.'''
    havePick = False
    picklock = None
    for tool in ch.inv + ch.eq:
        if tool.name == 'wytrych':
            havePick = True
            break
    if not havePick:
        ch.send("Nie masz wytrychu aby próbować otworzyć " + objname + ".")
        return False
    for tool in ch.eq:
        if tool.name == 'wytrych':
            picklock = tool
            break
    if picklock == None:
        ch.send("Nie masz wytrychu w ręce aby próbować otworzyć " + objname + ".")
        return False
    #count chance to picklock object
    chroll = ch.aux('stats').values[1] + random.randint(0, 99)
    hasSkill = -1
    if 'Otwieranie zamków' in ch.aux('skills').names:
        hasSkill = ch.aux('skills').names.index('Otwieranie zamków')
        chroll += ch.aux('skills').levels[hasSkill]
    if chroll >= chance:
        if hasSkill > -1:
            ch.aux('skills').exp[hasSkill] += chance
        else:
            ch.aux('skills').names.append('Otwieranie zamków')
            ch.aux('skills').levels.append(0)
            ch.aux('skills').exp.append(chance)
        gainlevel.check_exp(ch, 'Otwieranie zamków', 'skill')
        ch.aux('stats').secondvalues[1] += chance
        gainlevel.check_exp(ch, 'dex', 'stat')
        hooks.run("durability", hooks.build_info("ch obj", (ch, picklock)))
        return True
    else:
        if hasSkill > -1:
            ch.aux('skills').exp[hasSkill] += 1
            gainlevel.check_exp(ch, 'Otwieranie zamków', 'skill')
        else:
            ch.aux('skills').names.append('Otwieranie zamków')
            ch.aux('skills').levels.append(0)
            ch.aux('skills').exp.append(1)
        ch.aux('stats').secondvalues[1] += 1
        gainlevel.check_exp(ch, 'dex', 'stat')
        hooks.run("durability", hooks.build_info("ch obj", (ch, picklock)))
        return False

def cmd_picklock(ch, cmd, arg):
    '''Usage: picklock <door | direction | container>

    Attempts to unlock the specified door, direction or container with picklock.'''
    try:
        found,type=mud.parse_args(ch, True, cmd, arg, "{obj.room.inv exit }")
    except: return
    # what did we find?
    if type == "exit":
        ex = found
        name = ex.name
        if ex.name == "":
            name = "wyjście"
        
        if not ex.is_closed:
            ch.send(name + " są już otwarte.")
        elif not ex.is_locked:
            ch.send(name + " są już odblokowane.")
        else:
            if do_picklock(ch, name, ex.pick_diff):
                mud.message(ch, None, None, None, True, "to_char",
                            "Odblokowujesz " + name + ".")
                mud.message(ch, None, None, None, True, "to_room",
                            "$n odblokowuje " + name + ".")
                ex.unlock()
                try_manip_other_exit(ch.room, ex, ex.is_closed, False)
            else:
                ch.send("Nie udało ci się otworzyć " + name + ".")

    # must be an object
    else:
        obj = found
        if not obj.istype("container"):
            ch.send(ch.see_as(obj) + " nie jest pojemnikiem.")
        elif not obj.container_is_closed:
            ch.send(ch.see_as(obj) + " jest już otwarte.")
        elif not obj.container_is_locked:
            ch.send(ch.see_as(obj) + " jest już odblokowane.")
        else:
            if do_picklock(ch, obj.name, obj.container_pick_diff):
                mud.message(ch, None, obj, None, True, "to_char", "Odblokowujesz $o.")
                mud.message(ch, None, obj, None, True, "to_room", "$n odblokowuje $o.")
                obj.container_is_locked = False
            else:
                ch.send("Nie udało ci się otworzyć " + obj.name + ".")
    

def cmd_unlock(ch, cmd, arg):
    '''Usage: unlock <door | direction | container>

       Attempts to unlock the specified door, direction, or container.'''
    try:
        found,type=mud.parse_args(ch,True,cmd,arg, "{obj.room.inv exit }")
    except: return

    # what did we find?
    if type == "exit":
        ex = found
        name = ex.name
        if ex.name == "":
            name = "wyjście"
        
        if not ex.is_closed:
            ch.send(name + " są już otwarte.")
        elif not ex.is_locked:
            ch.send(name + " są już odblokowane.")
        elif ex.key == '':
            ch.send("Nie masz pojęcia jak odblokować " + name + ".")
        elif not utils.has_proto(ch, ex.key):
            ch.send("Nie możesz znaleźć klucza.")
        else:
            mud.message(ch, None, None, None, True, "to_char",
                        "Odblokowujesz " + name + ".")
            mud.message(ch, None, None, None, True, "to_room",
                        "$n odblokowuje " + name + ".")
            ex.unlock()
            # hooks.run("room_change", hooks.build_info("rm", (ch.room, )))
            try_manip_other_exit(ch.room, ex, ex.is_closed, False)

    # must be an object
    else:
        obj = found
        if not obj.istype("container"):
            ch.send(ch.see_as(obj) + " nie jest pojemnikiem.")
        elif not obj.container_is_closed:
            ch.send(ch.see_as(obj) + " jest już otwarte.")
        elif not obj.container_is_locked:
            ch.send(ch.see_as(obj) + " jest już odblokowane.")
        elif obj.container_key == '':
            ch.send("Nie masz pojęcia jak odblokować "+ ch.see_as(obj))
        elif not utils.has_proto(ch, obj.container_key):
            ch.send("Nie możesz znaleźć klucza.")
        else:
            mud.message(ch, None, obj, None, True, "to_char", "Odblokowujesz $o.")
            mud.message(ch, None, obj, None, True, "to_room", "$n odblokowuje $o.")
            obj.container_is_locked = False

def cmd_open(ch, cmd, arg):
    '''Usage: open [the] <direction | door | container>

       Attempts to open the speficied door, direction, or container.'''
    try:
        found,type=mud.parse_args(ch,True,cmd,arg, "{obj.room.inv exit }")
    except: return

    # is it an exit?
    if type == "exit":
        ex = found
        name = ex.name
        if name == "":
            name = "wyjście"
        
        if not ex.is_closed:
            ch.send(name + " są już otwarte.")
        elif ex.is_locked:
            ch.send(name + " muszą być najpierw odblokowane.")
        elif not ex.is_closable:
            ch.send(name + " nie mogą być otwarte.")
        else:
            mud.message(ch, None, None, None, True, "to_char",
                        "Otwierasz " + name + ".")
            mud.message(ch, None, None, None, True, "to_room",
                        "$n otwiera " + name + ".")
            ex.open()
            # hooks.run("room_change", hooks.build_info("rm", (ch.room, )))
            try_manip_other_exit(ch.room, ex, False, ex.is_locked)
            hooks.run("open_door", hooks.build_info("ch ex", (ch, ex)))

    # must be an object
    else:
        obj = found
        if not obj.istype("container"):
            ch.send(ch.see_as(obj) + " nie jest pojemnikiem.")
        elif not obj.container_is_closed:
            ch.send(ch.see_as(obj) + " jest już otwarty.")
        elif obj.container_is_locked:
            ch.send(ch.see_as(obj) + " musi być najpierw odblokowany.")
        elif not obj.container_is_closable:
            ch.send(ch.see_as(obj) + " nie może być otwarty.")
        else:
            mud.message(ch, None, obj, None, True, "to_char", "Otwierasz $o.")
            mud.message(ch, None, obj, None, True, "to_room", "$n otwiera $o.")
            obj.container_is_closed = False
            hooks.run("open_obj", hooks.build_info("ch obj", (ch, obj)))

def cmd_close(ch, cmd, arg):
    '''Usage: close <direction | door | container>

       Attempts to close the specified door, direction, or container.'''
    try:
        found,type=mud.parse_args(ch,True,cmd,arg, "{obj.room.inv exit }")
    except: return

    # is it an exit?
    if type == "exit":
        ex = found
        name = ex.name
        if name == "":
            name = "wyjście"

        if ex.is_closed:
            ch.send(name + " są już zamknięte.")
        elif ex.is_locked:
            ch.send(name + " muszą być odblokowane najpierw.")
        elif not ex.is_closable:
            ch.send(name + " nie mogą być zamknięte.")
        else:
            mud.message(ch, None, None, None, True, "to_char",
                        "Zamykasz " + name + ".")
            mud.message(ch, None, None, None, True, "to_room",
                        "$n zamyka " + name + ".")
            ex.close()
            try_manip_other_exit(ch.room, ex, True, ex.is_locked) 
            hooks.run("close_door", hooks.build_info("ch ex", (ch, ex)))

    # must be an object
    else:
        obj = found
        if not obj.istype("container"):
            ch.send(ch.see_as(obj) + " nie jest pojemnikiem.")
        elif obj.container_is_closed:
            ch.send(ch.see_as(obj) + " jest już zamknięty.")
        elif not obj.container_is_closable:
            ch.send(ch.see_as(obj) + " nie może być zamknięty.")
        else:
            mud.message(ch, None, obj, None, True, "to_char", "Zamykasz $o.")
            mud.message(ch, None, obj, None, True, "to_room", "$n zamyka $o.")
            obj.container_is_closed = True
            hooks.run("close_obj", hooks.build_info("ch obj", (ch, obj)))



################################################################################
# load all of our commands
################################################################################
mudsys.add_cmd("daj",   None, cmd_give,   "player", True)
mudsys.add_cmd("wez",    None, cmd_get,    "player", True)
mudsys.add_cmd("upusc",   None, cmd_drop,   "player", True)
mudsys.add_cmd("zdejmij", None, cmd_remove, "player", True)
mudsys.add_cmd("zaloz",   None, cmd_wear,   "player", True)
mudsys.add_cmd("wloz",    None, cmd_put,    "player", True)
mudsys.add_cmd("otworz",   None, cmd_open,   "player", True)
mudsys.add_cmd("zamknij",  None, cmd_close,  "player", True)
mudsys.add_cmd("zablokuj",   None, cmd_lock,   "player", True)
mudsys.add_cmd("odblokuj", None, cmd_unlock, "player", True)
mudsys.add_cmd("poswiec", None, cmd_sacrifice, "player", True)
mudsys.add_cmd("wlam", None, cmd_picklock, "player", True)

def chk_can_manip(ch, cmd):
    if not ch.pos in ["sitting", "standing", "flying"]:
        if ch.pos == "sleeping":
            pozycja = "śpisz"
        if ch.pos == "unconscious":
            pozycja = "jesteś nieprzytomny"
        ch.send("Nie możesz tego zrobić kiedy " + pozycja + ".")
        return False

for cmd in ["daj", "wez", "upusc", "zdejmij", "zaloz", "wloz", "otworz", "zamknij",
            "zablokuj", "odblokuj", "poswiec", "wlam"]:
    mudsys.add_cmd_check(cmd, chk_can_manip)

#add bits for shops
mudsys.create_bit("char_prfs", "shop")
mudsys.create_bit("char_prfs", "bank")
mudsys.create_bit("obj_bits", "nosac")

#add picklock
mudsys.add_worn_type('tool', 'dzierżone')
