# -*- coding: utf-8 -*-
'''
cmd_misc.c

a collection of miscellaneous commands that come with NakedMud(tm)
'''
import mudsys, mud, hooks, event, mudsys



def cmd_stop(ch, cmd, arg):
    '''If you are currently performing an action (for example, a delayed
       command), make an attempt to stop performing that action.'''
    if not ch.isActing():
        ch.send("Ale nie wykonujesz obecnie żadnej akcji!\r\n")
    else:
        ch.interrupt()

def cmd_clear(ch, cmd, arg):
    '''This command will clear your display screen.'''
    ch.send_raw("\033[H\033[J")

def event_delayed_cmd(ch, filler, cmd):
    '''used to perform delayed commands'''
    ch.act(cmd, True)

def cmd_delay(ch, cmd, arg):
    '''Usage: delay <seconds> <command>

       Allows the user to prepare a command to be executed in the future. For
       example:

       > delay 2 say hello, world!

       Will make you say \'hello, world!\' two seconds after entering the
       delayed command.'''
    try:
        secs, to_delay = mud.parse_args(ch, True, cmd, arg, "double string")
    except: return

    if secs < 1:
        ch.send("Możesz opóźniać komendy tylko o dodatnią ilość czasu.")
    else:
        ch.send("Opóźniasz wykonanie '%s' na %.2f sekund" % (to_delay, secs))
        event.start_event(ch, secs, event_delayed_cmd, None, to_delay)

def cmd_motd(ch, cmd, arg):
    '''This command will display to you the mud\'s message of the day.'''
    ch.page(mud.get_motd())

def cmd_save(ch, cmd, arg):
    '''Attempt to save your character and all recent changes made to it, to
       disk. This automatically happens when logging out.'''
    if mudsys.do_save(ch):
        ch.send("Twoja postać została zapisana.")
    else:
        ch.send("Twoja postać nie została zapisana.")
    

def cmd_quit(ch, cmd, arg):
    '''Attempts to save and log out of the game.'''
    mud.log_string(ch.name + " has left the game.")
    if ch.name.lower().startswith("gość"):
        mudsys.do_disconnect(ch)
    else:
        mudsys.do_save(ch)
        mudsys.do_quit(ch)

def finish_desc_editor(sock, editor_output):
    '''this function is passed to edit_text, and called after a socket exits the
       text editor.'''
    if sock.ch != None:
        sock.ch.desc = editor_output

def cmd_describe(ch, cmd, arg):
    '''Allows a character to set his or her own description.'''
    if ch.sock == None:
        ch.send("Wygląda na to, że utraciłeś połączenie.")
    else:
        ch.sock.edit_text(ch.desc, finish_desc_editor)

def cmd_rules(ch, cmd, arg):
    '''Usage: zasady [opcja]

    Show game rules. If additional [opcja] is specified, show specified rule.'''
    if arg == "":
        ch.act("pomoc prawa anarii")
    else:
        try:
            number = mud.parse_args(ch, True, cmd, arg, "int")
        except:
            return
        ch.act("pomoc zasada" + str(arg))

################################################################################
# add our commands
################################################################################
mudsys.add_cmd("stop",  None, cmd_stop,  "player", False)
mudsys.add_cmd("wyczysc", None, cmd_clear, "player", False)
mudsys.add_cmd("opoznij", None, cmd_delay, "player", False)
mudsys.add_cmd("motd",  None, cmd_motd,  "player", False)
mudsys.add_cmd("zapisz",  None, cmd_save,  "player", False)
mudsys.add_cmd("wyjdz",  None, cmd_quit,  "player", True)
mudsys.add_cmd("opis", None, cmd_describe, "player", True)
mudsys.add_cmd("zasady", None, cmd_rules, "player", True)

chk_can_save = lambda ch, cmd: not ch.is_npc
mudsys.add_cmd_check("zapisz", chk_can_save)
mudsys.add_cmd_check("wyjdz", chk_can_save)
