# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

import mud, mudsys, auxiliary, storage

################################################################################
# define cooldown data
################################################################################

class CooldownData:
    '''Data structure for cooldowns.'''

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.names = []
        self.times = []

        if set != None and len(set.readString("names")) > 0:
            self.names = set.readString("names").split(';')
            if len(self.names) == 0:
                return
            times = set.readString("times").split(';')
            for time in times:
                self.times.append(int(time))

    def copyTo(self, to):
        to.names = self.names
        to.times = self.times

    def copy(self):
        newData = CooldownData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString("names", ';'.join(self.names))
        values = []
        for value in self.times:
            values.append(str(value))
        set.storeString("times", ';'.join(values))
        return set

################################################################################
# initialization
################################################################################
auxiliary.install("cooldowns", CooldownData, "character")
