# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
cron.py

Different automatic tasks for whole MUD.
'''
import os, hooks, event, mudsys, time, account, obj, utils, mud, char, mudsock

################################################################################
# events
################################################################################

def purgeacc_event(owner, void, info):
    '''event for purge unused accounts.'''
    dirs = os.listdir('accounts')
    deltime = time.time() - 7776000
    for alpha in dirs:
        accounts = os.listdir('accounts/' + alpha)
        if len(accounts) > 0:
            for acct in accounts:
                acc = mudsys.load_account(acct[0:-5])
                if acc.aux('data').lastvisited <= deltime:
                    chars = acc.characters()
                    if len(chars) > 0:
                        for char in chars:
                            os.unlink("players/pfiles/" + char[0] + "/" + char + ".pfile")
                            os.unlink("players/objfiles/" + char[0] + "/" + char + ".ofile")
                    os.unlink('accounts/' + alpha + "/" + acct)
    event.start_event(None, 86400, purgeacc_event, None, '')

def purgeobj_event(owner, void, info):
    '''event for autopurge orphaned items.'''
    for item in obj.obj_list():
        if utils.is_keyword(item.bits, "nopurge"):
            continue
        if item.carrier == None and item.wearer == None and not item.hasvar('owner') and item.container == None and item.age > 300:
            mud.extract(item)
    event.start_event(None, 300, purgeobj_event, None, '')

def autosave_event(owner, void, info):
    '''Function autosave characters every 15 minutes'''
    for ch in char.char_list():
        if ch.is_npc or ch.name.lower().startswith('gość'):
            continue
        mudsys.do_save(ch)
    event.start_event(None, 900, autosave_event, None, '')

def cooldown_event(owner, void, info):
    '''event for cooldowns of skills.'''
    for ch in char.char_list():
        if len(ch.aux('cooldowns').names) == 0:
            continue
        i = 0
        length = len(ch.aux('cooldowns').names)
        while i < length:
            if ch.aux('cooldowns').times[i] <= 0:
                if ch.is_pc and not ch.aux('cooldowns').names[i].startswith('*'):
                    ch.send("Możesz ponownie " + ch.aux('cooldowns').names[i] + ".")
                del ch.aux('cooldowns').names[i]
                del ch.aux('cooldowns').times[i]
                length = len(ch.aux('cooldowns').names)
            else:
                ch.aux('cooldowns').times[i] -= 1
            i += 1
    event.start_event(None, 1, cooldown_event, None, '')

def disconnect_event(owner, void, info):
    '''event for disconnect inactive connections.'''
    for sock in mudsock.socket_list():
        if sock.idle_time >= 600:
            if sock.account != None:
                if sock.ch != None:
                    mudsys.do_save(sock.ch)
                    mudsys.do_quit(sock.ch)
                mudsys.do_save(sock.account)
            sock.close()
    event.start_event(None, 600, disconnect_event, None, '')
    
def weather_event(owner, void, info):
    '''event for weather.'''
    hooks.run('weather', hooks.build_info("int", (1, )))
    event.start_event(None, 300, weather_event, None, '')

################################################################################
# hooks
################################################################################

def purgeacc_hook(info):
    '''hook for autopurge unused accounts.'''
    event.start_event(None, 86400, purgeacc_event, None, '')

def purgeobj_hook(info):
    '''hook for autopurge orphaned items.'''
    event.start_event(None, 300, purgeobj_event, None, '')

def autosave_hook(info):
    '''autosave characters over time'''
    event.start_event(None, 900, autosave_event, None, '')

def cooldown_hook(info):
    '''cooldown skills.'''
    event.start_event(None, 1, cooldown_event, None, '')

def disconnect_hook(info):
    '''disconnect dead links.'''
    event.start_event(None, 600, disconnect_event, None, '')

def weather2_hook(info):
    '''count weather for each zone.'''
    event.start_event(None, 300, weather_event, None, '')

################################################################################
# initialization and unloading
################################################################################
hooks.add("start", purgeacc_hook)
hooks.add("start", purgeobj_hook)
hooks.add("start", autosave_hook)
hooks.add("start", cooldown_hook)
hooks.add("start", disconnect_hook)
hooks.add("start", weather2_hook)

mudsys.create_bit("obj_bits", "nopurge")

def __unload__():
    '''things that need to be detached when the module is un/reloaded'''
    hooks.remove("start", purgeacc_hook)
    hooks.remove("start", purgeobj_hook)
    hooks.remove("start", autosave_hook)
    hooks.remove("start", cooldown_hook)
    hooks.remove("start", disconnect_hook)
    hooks.remove("start", weather2_hook)
