# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
damage.py

Various function related to damage received by character.
'''
import obj, char, mud, event, odmiana, random

################################################################################
# functions
################################################################################

def create_corpse(owner, killer, type):
    '''Create corpse of killed character and put there all items.'''
    #first, create object and set it values
    corpse = obj.load_obj('zwloki1@matherion', owner.room)
    corpse.keywords = 'zwloki, zwłoki, ' + owner.name
    corpse.name = odmiana.zastap_odmiane('Zwłoki ' + owner.name + '$g', owner)
    corpse.rdesc = odmiana.zastap_odmiane('Zwłoki ' + owner.name + '$g leżą tutaj.', owner)
    if owner.is_npc:
        corpse.worth = owner.worth
        if type == 'char':
            corpse.setvar('owner', killer)
    else:
        owner.send("GINIESZ!")
        owner.aux('alignment').value = 0
        owner.aux('alignment').desc = 'Neutralny'
        corpse.setvar('owner', owner.name)
    #second, get all things from owner and put it to corpse
    for item in owner.eq:
        item.carrier = owner
    for item in owner.inv:
        item.container = corpse
    #if owner was npc, remove him from game, otherwise teleport to church
    if owner.is_npc:
        mud.extract(owner)
    else:
        church, = mud.parse_args(owner, True, '', 'katedra4@matherion', 'room')
        owner.room = church
        owner.position = "unconscious"
    event.start_event(corpse, 600, remove_corpse_event, None, '')

def get_armor(ch):
    '''Get armor and body locations of character. Returns list with: Armor locations, Hit Armor locations and Body locations'''
    chArmor = {}
    chArmorHit = {}
    chBody = {}
    for bpart in ch.bodyparts:
        size = int(ch.get_bodypct(bpart + ',') * 100)
        chBody[bpart] = size
        obj = ch.get_equip(bpart)
        if obj != None:
            if obj.istype('armor'):
                if bpart == 'trzymane':
                    chArmorHit[bpart] = False
                    continue
                chArmor[bpart] = obj.get_type_data('armor').resistance
                chArmorHit[bpart] = False
    return [chArmor, chArmorHit, chBody]

def count_damage(body, armor, dmg, damagetype):
    '''Return damage received, body part damaged and information about hit in armor.'''
    #count body part which was hit
    hitroll = random.randint(1, 100)
    hitloc = 100
    for bpart in body.keys():
        hitloc -= body[bpart]
        if hitloc <= hitroll:
            hitpart = bpart
            break
    #did this body part is covered by armor?
    hitArmor = False
    if hitpart in armor.keys():
        damageResist = armor[hitpart]
        hitArmor = True
    else:
        damageResist = 0
    #count damage received by defender
    if damageResist > 0:
        dmg -= (dmg * float(damageResist / 100))
    if dmg < 0:
        dmg = 0
    if dmg > 0:
        if damagetype == 'cut':
            dmg = dmg * 1.5
        if damagetype == 'impaling':
            dmg = dmg * 2
    return [dmg, hitpart, hitArmor]

def check_armor(ch, armorHit):
    '''Check armor parts for damage.'''
    for bpart in armorHit.keys():
        if armorHit[bpart]:
            hooks.run("durability", hooks.build_info("ch obj", (ch, ch.get_equip(bpart))))

################################################################################
# game events
################################################################################

def remove_corpse_event(owner, data, arg):
    '''Remove corpse from game after 10 minutes.'''
    if owner == None:
        return
    mud.extract(owner)
