# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria MUD.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
durability.py

Provide data structure and functions for durability of items
'''

import mudsys, mud, auxiliary, storage, olc, hooks, utils, random, char, gainlevel

################################################################################
# define durability data
################################################################################
materials = []

class DurabilityData:
    '''Data structure for durability.'''

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.durability = 100
        self.maxdurability = 100
        self.material = "z drewna"

        if set != None:
            self.durability = set.readInt('durability')
            self.maxdurability = set.readInt('maxdurability')
            if set.contains('material'):
                self.material = set.readString('material')

    def copyTo(self, to):
        to.durability = self.durability
        to.maxdurability = self.maxdurability
        to.material = self.material

    def copy(self):
        newData = DurabilityData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeInt('durability', self.durability)
        set.storeInt('maxdurability', self.maxdurability)
        set.storeString('material', self.material)
        return set

################################################################################
# durability OLC
################################################################################
__DUROLC_DUR__    = 1
__DUROLC_MAXDUR__ = 2
__DUROLC_MAT__    = 3

durolc_opt_map = {
    '1' : (__DUROLC_DUR__,   "Wprowadź obecną wytrzymałość: "),
    '2' : (__DUROLC_MAXDUR__, "Wprowadź maksymalną wytrzymałość: "),
    '3' : (__DUROLC_MAT__, "Wybierz materiał z którego wykonany jest obiekt: ")
    }

def durolc_menu(sock, data):
    '''Show durability of obiect.'''
    buf = []
    buf.append("{g1) Obecna wytrzymałość: {c%d{g" % data.aux('durability').durability)
    buf.append("{g2) Maksymalna wytrzymałość: {c%d{g" % data.aux('durability').maxdurability)
    buf.append("{g3) Wykonany: {c%s{g" % data.aux('durability').material)
    sock.send("\r\n".join(buf))

def durolc_chooser(sock, data, option):
    '''Choice option to change.'''
    if not durolc_opt_map.has_key(option):
        return olc.MENU_CHOICE_INVALID
    ret, mssg = durolc_opt_map[option]

    #list materials
    opts = None
    if ret == __DUROLC_MAT__:
        opts = materials

    if opts != None:
        utils.olc_display_table(sock, opts, 4)
    sock.send_raw(mssg)
    return ret

def durolc_parser(sock, owner, choice, arg):
    '''Set new value for durablity'''
    try:
        arg = int(arg)
    except:
        return False

    if choice == __DUROLC_DUR__:
        owner.aux('durability').durability = max(int(arg), 1)
        
    elif choice == __DUROLC_MAXDUR__:
        owner.aux('durability').maxdurability = max(int(arg), 1)

    elif choice == __DUROLC_MAT__:
        arg = int(arg)
        if arg < 0 or arg >= len(materials):
            return False
        owner.aux('durability').material = materials[arg]
    else:
        return False
    return True

def dur_to_proto(owner):
    '''Save durability to file.'''
    buf = 'me.aux("durability").durability = %d\n' % owner.aux('durability').durability
    buf += 'me.aux("durability").maxdurability = %d\n' % owner.aux('durability').maxdurability
    buf += 'me.aux("durability").material = "%s"\n' % owner.aux('durability').material
    return buf

def durolc_extend_menu(sock, data):
    '''Show option in main menu'''
    sock.send("Wytrzymałość i materiał")

def durolc_choice_exec(sock, data):
    '''Edit durability'''
    olc.do_olc(sock, durolc_menu, durolc_chooser, durolc_parser, None, data)
    return olc.MENU_NOCHOICE

################################################################################
# functions
################################################################################

def check_dur(ch, obj):
    '''Function check durability of object after using it. If object
    durability = 0, destroy it.'''
    if utils.is_keyword(obj.bits, "nodestroy"):
        return
    chances = [50, 25, 10, 75, 5, 30, 40, 60, 100, 30]
    material = materials.index(obj.aux('durability').material)
    chance = random.randint(0, 99)
    if chance <= chances[material]:
        curdur = obj.aux('durability').durability - 1
        if curdur == 0:
            ch.send(obj.name + " ulega zniszczeniu!")
            mud.extract(obj)
        else:
            obj.aux('durability').durability = curdur

################################################################################
# game commands
################################################################################

def cmd_repair(ch, cmd, arg):
    '''Usage: reperuj <przedmiot>

    Repair selected item.'''
    if 'naprawiać przedmioty' in ch.aux('cooldowns').names:
        ch.send('Przed chwilą naprawiałeś przedmiot. Odpocznij trochę zanim zrobisz to ponownie.')
        return
    try:
        item, = mud.parse_args(ch, True, cmd, arg, "obj.inv")
    except:
        return

    if item.aux('durability').material == "z ciała":
        ch.send(item.name + " nie może być naprawiany.")
        return
    if item.aux('durability').durability == item.aux('durability').maxdurability:
        ch.send(item.name + " jest w doskonałym stanie.")
        return
    haveTools = False
    for tool in ch.inv + ch.eq:
        if tool.name == 'zestaw narzędzi':
            tools = tool
            haveTools = True
            break
    if not haveTools:
        ch.send("Nie masz narzędzi aby naprawić " + item.name  + "$a.")
        return
    #start repairing item
    skills = { 'Stolarstwo' : ['z drewna'],
               'Kowalstwo' : ['z żelaza', 'ze stali', 'z miedzi'],
               'Garncarstwo' : ['z gliny', 'ze szkła'],
               'Kuśnierstwo' : ['ze skóry'],
               'Krawiectwo' : ['z materiału'],
               'Kamieniarstwo' : ['z kamienia']
               }
    skill = None
    for key in skills.keys():
        if item.aux('durability').material in skills[key]:
            skill = key
            break
    chroll = ch.aux('stats').values[1] + random.randint(0, 99)
    chance = item.aux('durability').maxdurability - item.aux('durability').durability
    hasSkill = -1
    if skill in ch.aux('skills').names:
        hasSkill = ch.aux('skills').names.index(skill)
        chroll += ch.aux('skills').levels[hasSkill]
    if chroll < chance:
        ch.send("Nie udało ci się naprawić " + item.name + "$g.")
    else:
        exp = max((chance / 10), 1)
        if hasSkill > -1:
            ch.aux('skills').exp[hasSkill] += exp
        else:
            ch.aux('skills').names.append(skill)
            ch.aux('skills').levels.append(0)
            ch.aux('skills').exp.append(exp)
        gainlevel.check_exp(ch, skill, 'skill')
        ch.aux('stats').secondvalues[1] += exp
        gainlevel.check_exp(ch, 'dex', 'stat')
        item.aux('durability').maxdurability -= 1
        item.aux('durability').durability = item.aux('durability').maxdurability
        ch.send("Naprawiłeś " + item.name + "$a.")
    check_dur(ch, tools)
    ch.aux('cooldowns').names.append('naprawiać przedmioty')
    ch.aux('cooldowns').times.append(60)

################################################################################
# game hooks
################################################################################
def dur_desc_info(info):
    '''appends information about durability of object to the description
       buffer when a person looks at it.'''
    obj, ch = hooks.parse_info(info)

    if obj.aux('durability').material != 'z ciała':
        desc = " Wykonany " + obj.aux('durability').material + " i jest "
    else:
        desc = " Jest "
    if utils.is_keyword(obj.bits, "nodestroy"):
        desc += "niezniszczalny."
    else:
        max_dur = obj.aux('durability').maxdurability
        if max_dur == 0:
            return
        percent = (float(obj.aux('durability').durability) / float(max_dur)) * 100
        if percent == 100:
            desc += "w doskonałym stanie."
        elif percent > 89 and percent < 100:
            desc += "w dobrym stanie."
        elif percent > 79 and percent < 90:
            desc += "lekko zabrudzony."
        elif percent > 69 and percent < 80:
            desc += "zabrudzony."
        elif percent > 59 and percent < 70:
            desc += "lekko porysowany."
        elif percent > 49 and percent < 60:
            desc += "porysowany."
        elif percent > 39 and percent < 50:
            desc += "mocno porysowany."
        elif percent > 29 and percent < 40:
            desc += "podniszczony."
        elif percent > 19 and percent < 30:
            desc += "popękany."
        elif percent > 9 and percent < 20:
            desc += "mocno uszkodzony."
        elif percent > 0 and percent < 10:
            desc += "niemal zniszczony."
    
    # append our description
    ch.look_buf += desc

def check_dur_hook(info):
    '''When thing was used, check if it was damaged or not.'''
    ch, item = hooks.parse_info(info)
    if ch.is_pc:
        check_dur(ch, item)

################################################################################
# intialization
################################################################################
auxiliary.install("durability", DurabilityData, "object")

olc.extend("oedit", "D", durolc_extend_menu, durolc_choice_exec, None, None,
           dur_to_proto)

# display hooks for durability
hooks.add("append_obj_desc", dur_desc_info)
hooks.add('durability', check_dur_hook)

# add obiect materials
materials.append('z drewna')
materials.append('z żelaza')
materials.append('ze stali')
materials.append('ze szkła')
materials.append('z kamienia')
materials.append('ze skóry')
materials.append('z materiału')
materials.append('z gliny')
materials.append('z ciała')
materials.append('z miedzi')

mudsys.create_bit("obj_bits", "nodestroy")
mudsys.create_bit("char_prfs", "repair")

#add commands
mudsys.add_cmd("reperuj", None, cmd_repair, "player", False)

def chk_can_repair(ch, cmd):
    if ch.pos in ["sleeping", "uncoscious"]:
        if ch.pos == "sleeping":
            pozycja = "śpisz"
        elif ch.pos == "unconscious":
            pozycja = "jesteś nieprzytomny"
        ch.send("Nie możesz możesz naprawiać rzeczy, kiedy " + pozycja + "!")
        return False

mudsys.add_cmd_check("reperuj", chk_can_repair)

def __unload__():
    '''things that need to be detached when the module is un/reloaded'''
    hooks.remove("append_obj_desc", dur_desc_info)
    hooks.remove('durability', check_dur_hook)
