# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
faith.py

Provide data structure and functions for players faith.
'''

import mudsys, mud, auxiliary, storage, hooks, olc

################################################################################
# faith data type definition
################################################################################

class FaithData:
    '''Faith character data.'''

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.deity = 'Brak'
        self.type = ''
        self.value = 0
        self.desc = ''
        self.minalign = -100
        self.maxalign = 100

        if set != None:
            if len(set.readString('deity')) > 0:
                self.deity = set.readString('deity')
                self.type = set.readString('type')
                self.value = set.readInt('value')
                self.desc = set.readString('desc')
                self.minalign = set.readInt('minalign')
                self.maxalign = set.readInt('maxalign')

    def copyTo(self, to):
        to.deity = self.deity
        to.type = self.type
        to.value = self.value
        to.desc = self.desc
        to.minalign = self.minalign
        to.maxalign = self.maxalign

    def copy(self):
        newData = FaithData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString('deity', self.deity)
        set.storeString('type', self.type)
        set.storeInt('value', self.value)
        set.storeString('desc', self.desc)
        set.storeInt('minalign', self.minalign)
        set.storeInt('maxalign', self.maxalign)
        return set

################################################################################
# functions
################################################################################

def count_faith(ch):
    '''Set description for level of faith.'''
    #no point in continue
    if ch.aux('faith').deity == 'Brak':
        return

    if ch.aux('faith').value > 50:
        desc = 'Umiłowany'
    elif ch.aux('faith').value > 0 and ch.aux('faith').value < 51:
        desc = 'Pozytywny'
    elif ch.aux('faith').value == 0:
        desc = 'Neutralny'
    elif ch.aux('faith').value < 0 and ch.aux('faith').value > -51:
        desc = 'Niechętny'
    elif ch.aux('faith').value < -50 and ch.aux('faith').value > -101:
        desc = 'Znienawidzony'
    else:
        desc = ''
        ch.aux('faith').deity = 'Brak'
        ch.aux('faith').value = -100
        ch.send("W związku z twoimi ostatnimi poczynaniami, twoje bóstwo, nie chce mieć więcej z tobą do czynienia.")
    if desc != ch.aux('faith').desc:
        if ch.aux('faith').deity != 'Brak':
            ch.send("Nastawienie twojego bóstwa do ciebie zmienia się.")
        ch.aux('faith').desc = desc

def pray_heal(ch, tgt, amount):
    '''Prayers which heal selected target by amount of hit points.'''
    curheal = tgt.aux('stats').values[4]
    maxheal = tgt.aux('stats').secondvalues[4]
    healed = max(float(maxheal / 100) * amount, 1)
    if curheal + healed > maxheal:
        tgt.aux('stats').values[4] = maxheal
    else:
        tgt.aux('stats').values[4] += healed
    if ch.name == tgt.name:
        ch.send(ch.aux('faith').deity + " wysłuchał twej modlitwy i uleczył ciebie.")
    else:
        if ch.aux('faith').deity == tgt.aux('faith').deity:
            gain = 2
        else:
            gain = 1
        ch.aux('faith').value += gain
        ch.send(ch.aux('faith').deity + " wysłuchał twej modlitwy i uleczył " + tgt.name + "$d.")
        if tgt.is_pc:
            tgt.send(ch.aux('faith').deity + " wysłuchał modlitwy " + ch.name + "$a i uleczył ciebie.")

################################################################################
# faith OLC
################################################################################

def faitholc_menu(sock, data):
    '''Display faith settings.'''
    buf = []
    buf.append("{g1) Bóstwo                 : {c" + data.aux('faith').deity)
    buf.append("{g2) Typ bóstwa             : {c" + data.aux('faith').type)
    buf.append("{g3) Nastawienie bóstwa     : {c" + str(data.aux('faith').value))
    buf.append("{g   Opis                   : {c" + data.aux('faith').desc)
    buf.append("{g4) Min wartość charakteru : {c" + str(data.aux('faith').minalign))
    buf.append("{g5) Max wartość charakteru : {c" + str(data.aux('faith').maxalign))
    sock.send("\r\n".join(buf))

def faitholc_chooser(sock, data, option):
    '''Try to pick faith settings to set.'''
    try:
        opt = int(option) - 1
        if opt < 0 or opt > 4:
            return olc.MENU_CHOICE_INVALID
        sock.send("Wprowadź nową wartość: ", newline = False)
        return opt + 1
    except:
        return olc.MENU_CHOICE_INVALID

def faitholc_parser(sock, mobile, choice, arg):
    '''Set new value for faith settings.'''
    try:
        option = int(choice)
    except:
        return False
    if option > 2:
        try:
            arg = int(arg)
        except:
            return False
    
    if option == 1:
        mobile.aux('faith').deity = arg
    elif option == 2:
        mobile.aux('faith').type = arg
    elif option == 3:
        mobile.aux('faith').value = arg
        count_faith(mobile)
    elif option == 4:
        mobile.aux('faith').minalign = arg
    elif option == 5:
        mobile.aux('faith').maxalign = arg
    else:
        return False
    return True

def faitholc_extend_menu(sock, data):
    '''Display option in main menu.'''
    sock.send("Edytuj wiarę")

def faitholc_choice_exec(sock, data):
    '''Edit faith.'''
    olc.do_olc(sock, faitholc_menu, faitholc_chooser, faitholc_parser, None, data)
    return olc.MENU_NOCHOICE

################################################################################
# game commands
################################################################################

def cmd_pray(ch, cmd, arg):
    '''Pray to selected god.'''
    #if character don't have god, cannot use this command
    if ch.aux('faith').deity == 'Brak':
        ch.send("Nie wyznajesz jakiegokolwiek bóstwa, więc nie możesz używać tej komendy.")
        return
    #prayers available for all gods
    allprayers = [['charakter', 5, 'pozwala poznać charakter danej istoty'], ['przywrocenie', 10, 'sprowadza zwłoki postaci']]
    #prayers available for selected types of gods
    godpowers = {"leczące" : [['namaszczenie', 10, 'leczy lekkie obrażenia', 25],
                              ['uzdrowienie', 20, 'leczy średnie obrażenia', 50],
                              ['uleczenie', 30, 'leczy całkowicie obrażenia', 100]]
                 }
    lprayer = arg.split()
    availprayers = allprayers + godpowers[ch.aux('faith').type]
    prayer = []
    if len(lprayer) > 0:
        for pray in availprayers:
            if lprayer[0] == pray[0]:
                prayer = pray
                break
    if len(prayer) == 0:
        buf = []
        buf.append("Dostępe modlitwy to:")
        for pray in availprayers:
            buf.append("%s (%s)" % (pray[0], pray[2]))
        ch.send("\r\n".join(buf))
        return
    if ch.aux('faith').value < pray[1]:
        ch.send("Nie możesz prosić o to swojego bóstwa, ponieważ nie jest ono do Ciebie aż tak przychylnie nastawione.")
        return
    if ch.aux('stats').values[5] + pray[1] > ch.aux('stats').secondvalues[5]:
        ch.send("Nie możesz prosić o to swojego bóstwa, ponieważ jesteś zbyt zmęczony.")
        return
    if prayer[0] == 'charakter':
        if len(lprayer) == 1:
            ch.send("Czyj charakter chcesz sprawdzić?")
            return
        else:
            try:
                tgt, = mud.parse_args(ch, True, cmd, lprayer[1], "ch.room.noself")
            except:
                return
            ch.send(tgt.name + " ma charakter: " + tgt.aux('alignment').desc)
    elif prayer[0] == 'przywrocenie':
        try:
            body, = mud.parse_args(ch, True, cmd, ch.name, "obj.world")
        except:
            return
        body.room = ch.room
        ch.send("Twoje zwłoki pojawiają się obok ciebie.")
    elif prayer[0] in ['namaszczenie', 'uzdrowienie', 'uleczenie']:
        if len(lprayer) == 1:
            pray_heal(ch, ch, prayer[3])
        else:
            try:
                tgt, = mud.parse_args(ch, True, cmd, lprayer[1], "ch.room.noself")
            except:
                return
            pray_heal(ch, tgt, prayer[3])
    ch.aux('faith').value -= pray[1]
    ch.aux('stats').values[5] += pray[1]
    count_faith(ch)

################################################################################
# game hooks
################################################################################

def faith_hook(info):
    '''When player faith value change, get new description for it.'''
    ch, = hooks.parse_info(info)
    if ch.is_pc:
        count_faith(ch)

################################################################################
# initialization
################################################################################
auxiliary.install("faith", FaithData, "character")

olc.extend("pcedit", "F", faitholc_extend_menu, faitholc_choice_exec, None, None, None)

mudsys.add_cmd("modlitwa", None, cmd_pray, "player", True)

mudsys.create_bit('char_prfs', 'priest')

def chk_can_pray(ch, cmd):
    if not ch.pos in ["sitting", "standing", "flying"]:
        if ch.pos == "sleeping":
            pozycja = "śpisz"
        if ch.pos == "unconscious":
            pozycja = "jesteś nieprzytomny"
        ch.send("Nie możesz modlić się kiedy " + pozycja + ".")
        return False

mudsys.add_cmd_check("modlitwa", chk_can_pray)

hooks.add('faith', faith_hook)

def __unload__():
    '''removes the hooks for faith.'''
    hooks.remove('faith', faith_hook)
