# -*- coding: utf-8 -*-
'''
follow.py

Follow after selected character
'''

import mudsys, char, mud, hooks

################################################################################
# game commands
################################################################################
def cmd_follow(ch, cmd, arg):
    '''Usage: podazaj <osoba>

    Start following after selected player.'''
    try:
        tgt, = mud.parse_args(ch, True, cmd, arg, "ch.room.noself")
    except:
        return
    if tgt.is_npc:
        ch.send("To nie jest gracz.")
        return
    ch.setvar("follow", tgt.name)
    ch.send("Zaczynasz podążać za " + tgt.name + "$i.")
    tgt.send(ch.name + " zaczyna podążać za tobą.")

def cmd_unfollow(ch, cmd, arg):
    '''Usage: stoj

    Stop following after selected player.'''
    if not ch.hasvar("follow"):
        ch.send("Nie podążasz za kimkolwiek.")
        return
    name = ch.getvar("follow")
    ch.delvar("follow")
    ch.send("Przestałeś podążać za " + name + "$i.")
    tgt = mudsys.get_player(name)
    tgt.send(ch.name + " przestał podążać za tobą.")

def cmd_dismiss(ch, cmd, arg):
    '''Usage: odpedz <osoba>

    Dismiss selected follower.'''
    try:
        tgt, = mud.parse_args(ch, True, cmd, arg, "ch.room.noself")
    except:
        return
    if not tgt.hasvar("follow"):
        ch.send(tgt.name + " nie podąża za tobą, więc nie możesz go odpędzić.")
    elif tgt.getvar("follow") != ch.name:
        ch.send(tgt.name + " nie podąża za tobą, więc nie możesz go odpędzić.")
    else:
        tgt.delvar("follow")
        ch.send("Odpędziłeś od siebie " + tgt.name + "$a.")
        tgt.send(ch.name + " odpędził ciebie.")

################################################################################
# game hooks
################################################################################
def follow_exit_hook(info):
    '''follow after selected player. If he exit from room, player exit too.'''
    ch, rm, ex = hooks.parse_info(info)
    if ch.is_npc:
        return
    dir_name = ["północ", "wschód", "południe", "zachód", "północny-wschód", "północny-zachód",
                 "południowy-zachód", "południowy-wschód", "góra", "dół"]
    dir_abbr = ["pn", "w", "pd", "z", "pnw", "pnz", "pdz", "pdw", "g", "d"]
    index = dir_name.index(rm.exdir(ex))
    # move followers after character
    for char in rm.chars:
        if char.hasvar("follow"):
            if char.getvar("follow") == ch.name:
                char.act(dir_abbr[index])

def follow_enter_hook(info):
    '''Remove followed player if he isn\'t in this same room which character enter.'''
    ch, rm = hooks.parse_info(info)
    if ch.hasvar("follow"):
        ishere = False
        for char in rm.chars:
            if char.name == ch.getvar("follow"):
                ishere = True
                break
        if not ishere:
            ch.act("stoj")

################################################################################
# initialization
################################################################################

# add commands
mudsys.add_cmd("podazaj", None, cmd_follow, "player", False)
mudsys.add_cmd("stoj", None, cmd_unfollow, "player", False)
mudsys.add_cmd("odpedz", None, cmd_dismiss, "player", False)

# exit hook for follow
hooks.add("exit", follow_exit_hook)
hooks.add("enter", follow_enter_hook)
