# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria MUD.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
hide.py

Hide/sneak/detect hidden functions
'''

import mudsys, char, random, gainlevel, mud

################################################################################
# functions
################################################################################

def can_see(ch, tgt):
    '''Check if character can see selected object.'''
    #if character have special flag, can see everything
    if ch.hasPrefs('seeall'):
        return True
    #did we are in dark room?
    if ch.room.hasBit('dark'):
        if not tgt in ch.eq + ch.inv:
            cansee = False
            for char in ch.room.chars:
                item = char.get_equip('trzymane')
                if item != None:
                    if item.istype('light') and item.hasvar('on'):
                        cansee = True
                        break
            if not cansee:
                return False
    #if object is visible, we simply see it
    if tgt.hidden == 0:
        return True
    #did character can see object
    chroll = ch.aux('stats').values[2] + random.randint(0, 99)
    hasSkill = -1
    if 'Spostrzegawczość' in ch.aux('skills').names:
        hasSkill = ch.aux('skills').names.index('Spostrzegawczość')
        chroll += ch.aux('skills').levels[hasSkill]
    if chroll >= tgt.hidden:
        if ch.is_pc and not '*Spostrzegawczość' in ch.aux('cooldowns').names:
            if hasSkill > -1:
                ch.aux('skills').exp[hasSkill] += 1
                gainlevel.check_exp(ch, 'Spostrzegawczość', 'skill')
            else:
                ch.aux('skills').names.append('Spostrzegawczość')
                ch.aux('skills').levels.append(0)
                ch.aux('skills').exp.append(1)
            ch.aux('stats').secondvalues[2] += 1
            gainlevel.check_exp(ch, 'int', 'stat')
            ch.aux('cooldowns').names.append('*Spostrzegawczość')
            ch.aux('cooldowns').times.append(10)
        return True
    else:
        return False

################################################################################
# game commands
################################################################################

def cmd_seeall(ch, cmd, arg):
    '''Command set or unset ability to see all things in game.'''
    if ch.hasPrefs('seeall'):
        if ch.prfs == 'seeall':
            ch.prfs = 'none'
        else:
            ch.prfs.replace('seeall', '')
        ch.send('Nie widzisz już ukrytych rzeczy.')
    else:
        ch.prfs += 'seeall'
        ch.send('Od teraz widzisz ukryte rzeczy.')

def cmd_hide(ch, cmd, arg):
    '''Usage: ukryj

    Hide self in shadows.'''
    if ch.hidden > 0:
        ch.send('Już się ukrywasz.')
        return
    if 'ukrywać się' in ch.aux('cooldowns').names:
        ch.send('Przed chwilą się ukrywałeś, poczekaj trochę zanim zrobisz to ponownie.')
        return
    item = ch.get_equip('trzymane')
    if item.istype('light') and hasvar('on'):
        ch.send('Nie możesz się ukrywać z zapaloną ' + item.name + '$l!')
        return
    chroll = ch.aux('stats').values[1] + random.randint(0, 99)
    hasSkill = -1
    if 'Ukrywanie się' in ch.aux('skills').names:
        hasSkill = ch.aux('skills').names.index('Ukrywanie się')
        chroll += ch.aux('skills').levels[hasSkill]
    ch.hidden = chroll
    if ch.is_pc:
        if hasSkill > -1:
            ch.aux('skills').exp[hasSkill] += 1
            gainlevel.check_exp(ch, 'Ukrywanie się', 'skill')
        else:
            ch.aux('skills').names.append('Ukrywanie się')
            ch.aux('skills').levels.append(0)
            ch.aux('skills').exp.append(1)
        ch.aux('stats').secondvalues[1] += 1
        gainlevel.check_exp(ch, "dex", "stat")
        ch.aux('cooldowns').names.append('ukrywać się')
        ch.aux('cooldowns').times.append(10)
        ch.send('Ukrywasz się wśród cieni.')
        mud.message(ch, None, None, None, True, 'to_room', '$n ukrywa się wśród cieni.')

def cmd_unhide(ch, cmd, arg):
    '''Usage: odkryj

    Unhide self.'''
    if ch.hidden == 0:
        ch.send('Nie ukrywasz się w tym momencie.')
    else:
        ch.hidden = 0
        ch.send('Wynurzasz się z cienia.')
        mud.message(ch, None, None, None, True, 'to_room', '$n wychodzi z ukrycia.')
        

################################################################################
# initialization
################################################################################
mudsys.add_cmd("seeall", None, cmd_seeall, "wizard", True)
mudsys.add_cmd("ukryj", None, cmd_hide, "player", True)
mudsys.add_cmd("odkryj", None, cmd_unhide, "player", True)

mudsys.create_bit('char_prfs', 'seeall')
mudsys.register_char_cansee(can_see)
mudsys.register_obj_cansee(can_see)
mudsys.register_exit_cansee(can_see)
