# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
light.py

Light item type.
'''
import olc, mudsys, auxiliary, mudsock, storage, mud, utils, hooks, event

################################################################################
# item type definition
################################################################################

light_type = []

class LightData:
    '''Data for light.'''

    __item_type__ = 'light'

    def __init__(self, set = None):
        self.type = 'pochodnia'
        self.usage = 6000
        self.maxusage = 6000

        if set != None:
            self.type = set.readString('type')
            self.usage = set.readInt('usage')
            self.maxusage = set.readInt('maxusage')

    def copyTo(self, to):
        to.type = self.type
        to.usage = self.usage
        to.maxusage = self.maxusage

    def copy(self):
        newData = LightData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString('type', self.type)
        set.storeInt('usage', self.usage)
        set.storeInt('maxusage', self.maxusage)
        return set

################################################################################
# Light OLC
################################################################################
__LIGHTOLC_TYPE__     = 1
__LIGHTOLC_USAGE__    = 2
__LIGHTOLC_MAXUSAGE__ = 3

lightolc_opt_map = {
    '1' : (__LIGHTOLC_TYPE__, 'Wybierz typ: '),
    '2' : (__LIGHTOLC_USAGE__, "Wprowadź czas używania: "),
    '3' : (__LIGHTOLC_MAXUSAGE__, "Wprowadź maksymalny czas używania: "),
    }

def lightolc_menu(sock, data):
    '''display our menu interface'''
    buf = []
    buf.append('{g1) Typ                       : {c' + data.type)
    buf.append('{g2) Czas używania             : {c' + str(data.usage))
    buf.append('{g3) Maksymalny czas używania  : {c' + str(data.maxusage))
    sock.send('\r\n'.join(buf))

def lightolc_chooser(sock, data, option):
    '''figure out what we want to edit, and display a prompt.'''
    if not lightolc_opt_map.has_key(option):
        return olc.MENU_CHOICE_INVALID
    ret, mssg = lightolc_opt_map[option]

    #list our types options
    if ret == __LIGHTOLC_TYPE__:
        utils.olc_display_table(sock, light_type, 4)
    sock.send_raw(mssg)
    return ret

def lightolc_parser(sock, data, choice, arg):
    '''parse out what we want to change a specified field to.'''
    try:
        arg = int(arg)
    except:
        return False

    if choice == __LIGHTOLC_TYPE__:
        arg = int(arg)
        if arg < 0 or arg >= len(light_type):
            return False
        newtype = light_type[arg]
        if newtype != data.type:
            data.type = newtype
            data.usage = 6000
            data.maxusage = 6000
    elif choice == __LIGHTOLC_USAGE__:
        data.usage = max(int(arg), 1)
    elif choice == __LIGHTOLC_MAXUSAGE__:
        data.maxusage = max(int(arg), 1)
    else:
        return False
    return True

def lightolc_to_proto(data):
    '''return a script that will generate relevant info to create this item.'''
    buf = 'me.get_type_data("light").type = "' + data.type + '"\n'
    buf += 'me.get_type_data("light").usage = ' + str(data.usage) + '\n'
    buf += 'me.get_type_data("light").maxusage = ' + str(data.maxusage) + '\n'
    return buf

################################################################################
# game hooks
################################################################################

def light_desc_info(info):
    '''appends light information about an object to the description buffer
    when a person looks at it.'''
    obj, ch = hooks.parse_info(info)

    # no point in continuing
    if not obj.istype('light'):
        return

    percent = (float(obj.get_type_data('light').usage) / float(obj.get_type_data('light').maxusage)) * 100
    desc = ' Poświeci jeszcze '
    if percent > 90:
        desc += 'bardzo długi czas.'
    elif percent < 91 and percent > 60:
        desc += 'długi czas.'
    elif percent < 61 and percent > 40:
        desc += 'jakiś czas.'
    elif percent < 41 and percent > 20:
        desc += 'krótki okres czasu.'
    elif percent < 20 and percent > 0:
        desc += 'trochę, ale niedługo zgaśnie.'
    elif percent == 0:
        desc = ' Już się nie zapali.'
    if obj.hasvar('on'):
        desc += ' Jest zapalona.'
    else:
        desc += ' Jest zgaszona.'

    # append out description
    ch.look_buf += desc

def turnon_light_hook(info):
    '''Turn on light source when player enter world.'''
    ch, = hooks.parse_info(info)
    if ch == None or not ch.is_pc:
        return
    item = ch.get_equip('trzymane')
    if item != None:
        if item.istype('light') and item.hasvar('on'):
            event.start_event(item, 1, light_event, None, '')

def turnoff_light_hook(info):
    '''Turn off light source when player remove it from self.'''
    ch, obj = hooks.parse_info(info)
    if obj.istype('light') and obj.hasvar('on'):
        obj.delvar('on')
        event.interrupt_events_involving(obj)

def dark_room_hook(info):
    '''Replace description of dark room.'''
    room, ch = hooks.parse_info(info)
    if room.hasBit('dark'):
        if ch.hasPrefs('seeall'):
            return
        item = ch.get_equip('trzymane')
        if item != None:
            if item.istype('light') and item.hasvar('on'):
                return
        ch.look_buf = 'Jest za ciemno by widzieć cokolwiek.'

################################################################################
# events
################################################################################

def light_event(owner, void, info):
    '''event for light source.'''
    owner.get_type_data('light').usage -= 1
    if owner.get_type_data('light').usage <= 0:
        owner.delvar("on")
        owner.wearer.send("Twoja " + owner.name + "$g gaśnie!")
        if owner.get_type_data('light').type == 'torch':
            mud.extract(owner)
    else:
        event.start_event(owner, 1, light_event, None, '')

################################################################################
# game commands
################################################################################

def cmd_lighton(ch, cmd, arg):
    '''Turn on light source.'''
    try:
        obj, = mud.parse_args(ch, True, cmd, arg, "obj.eq")
    except:
        return

    if not obj.istype('light'):
        ch.send("Nie możesz zapalić tego przedmiotu!")
    elif obj.hasvar('on'):
        ch.send(obj.name + ' jest już zapalona.')
    else:
        ch.send("Zapalasz " + obj.name + "$a.")
        obj.setvar("on", 1)
        event.start_event(obj, 1, light_event, None, '')

def cmd_lightoff(ch, cmd, arg):
    '''Turn off light source.'''
    try:
        obj, = mud.parse_args(ch, True, cmd, arg, "obj.eq")
    except:
        return

    if not obj.istype('light'):
        ch.send("Nie możesz zgasić tego przedmiotu!")
    elif not obj.hasvar('on'):
        ch.send(obj.name + ' jest już zgaszona.')
    else:
        ch.send("Gasisz " + obj.name + "$a.")
        obj.delvar("on")
        event.interrupt_events_involving(obj)

################################################################################
# initialization
################################################################################

#initialize our item data, and its OLC
mudsys.item_add_type(LightData.__item_type__, LightData)
olc.item_add_olc(LightData.__item_type__,
                 lightolc_menu, lightolc_chooser, lightolc_parser,
                 None, lightolc_to_proto)

#add hooks for light
hooks.add('append_obj_desc', light_desc_info)
hooks.add('char_to_game', turnon_light_hook)
hooks.add('unequip', turnoff_light_hook)
hooks.add('preprocess_room_desc', dark_room_hook)

#add commands
mudsys.add_cmd("zapal", None, cmd_lighton, "player", True)
mudsys.add_cmd("zgas", None, cmd_lightoff, "player", True)

#initialize some default light types
light_type.append('pochodnia')
light_type.append('latarnia')

#add dark rooms
mudsys.create_bit("room_bits", "dark")

def __unload__():
    '''remove all hooks.'''
    hooks.remove('append_obj_desc', light_desc_info)
    hooks.remove('char_to_game', turnon_light_hook)
    hooks.remove('unequip', turnoff_light_hook)
    hooks.remove('preprocess_room_desc', dark_room_hook)
