# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
mail.py

Provide data and functions for mail system.
'''

import mudsys, mud, auxiliary, storage, char

################################################################################
# mail data type definition
################################################################################

class Mail:
    '''One mail data structure.'''

    def __init__(self, sender, title, body, read, set = None):
        '''Standard class constructor.'''
        if set != None:
            self.sender = set.readString('sender')
            self.title = set.readString('title')
            self.body = set.readString('body')
            self.read = set.readString('read')
        else:
            self.sender = sender
            self.title = title
            self.body = body
            self.read = read

    def store(self):
        set = storage.StorageSet()
        set.storeString('sender', self.sender)
        set.storeString('title', self.title)
        set.storeString('body', self.body)
        set.storeString('read', self.read)
        return set

class MailData:
    '''Mail data.'''

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.mails = []

        if set != None:
            for mail in set.readList('mails').sets():
                self.mails.append(Mail(None, None, None, None, mail))

    def copyTo(self, to):
        to.mails = self.mails

    def copy(self):
        newData = MailData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        mails = storage.StorageList()
        set.storeList('mails', mails)
        for mail in self.mails:
            mails.add(mail.store())
        return set

################################################################################
# functions
################################################################################

def count_mails(ch):
    '''Function count all and unread mails of selected character.'''
    allmails = len(ch.aux('mail').mails)
    if allmails == 0:
        unread = 0
    else:
        unread = 0
        for mail in ch.aux('mail').mails:
            if mail.read == 'N':
                unread += 1
    return allmails, unread    

################################################################################
# game commands
################################################################################

def cmd_mail(ch, cmd, arg):
    '''Allow to read, write and delete mails.'''
    #if no argument, show how many mails player have
    if len(arg) == 0:
        allmails, unread = count_mails(ch)
        if allmails == 0:
            ch.send("Nie masz żadnych wiadomości.")
        elif allmails > 0 and unread == 0:
            ch.send("Masz %d wiadomości, wszystkie przeczytane." % (allmails,))
        else:
            ch.send('Masz %d wiadomości, z czego %d nieprzeczytanych.' % (allmails, unread))
        return
    options = arg.split()
    optlist = ['czytaj', 'wyslij', 'skasuj']
    if not options[0] in optlist:
        ch.send("Dostępne opcje to: czytaj, wyslij, skasuj")
        return
    if options[0] == 'czytaj':
        allmails, unread = count_mails(ch)
        if allmails == 0:
            ch.send("Nie masz żadnych wiadomości.")
            return
        if len(options) == 1:
            if unread == 0:
                buf = []
                buf.append("Dostępne wiadomości:")
                i = 1
                for mail in ch.aux('mail').mails:
                    buf.append('%d) Nadawca: %s Tytuł: %s' % (i, mail.sender, mail.title))
                    i += 1
                buf.append("Aby przeczytać wybraną wiadomość wpisz komendę poczta czytaj <numer>")
                ch.send("\r\n".join(buf))
            else:
                index = 0
                for mail in ch.aux('mail').mails:
                    if mail.read == 'N':
                        break;
                    index += 1
                buf = []
                buf.append("Nadawca: " + ch.aux('mail').mails[index].sender)
                buf.append("Tytuł: " + ch.aux('mail').mails[index].title)
                buf.append(ch.aux('mail').mails[index].body)
                ch.send("\r\n".join(buf))
                ch.aux('mail').mails[index].read = 'Y'
        else:
            try:
                index = int(options[1]) - 1
            except:
                ch.send("Nieprawidłowy numer wiadomości.")
                return
            if index > allmails or index < 0:
                ch.send("Nie masz takiej wiadomości.")
                return
            buf = []
            buf.append("Nadawca: " + ch.aux('mail').mails[index].sender)
            buf.append("Tytuł: " + ch.aux('mail').mails[index].title)
            buf.append(ch.aux('mail').mails[index].body)
            ch.send('\r\n'.join(buf))
            ch.aux('mail').mails[index].read = 'Y'
    elif options[0] == 'skasuj':
        allmails, unread = count_mails(ch)
        if len(options) < 2:
            ch.send("Musisz podać numer wiadomości do skasowania.")
            return
        try:
            index = int(options[1]) - 1
        except:
            ch.send("Nieprawidłowy numer wiadomości.")
            return
        if index > allmails or index < 0:
            ch.send("Nie masz takiej wiadomości.")
            return
        del ch.aux('mail').mails[index]
        ch.send("Skasowałeś wybraną wiadomość.")
    elif options[0] == 'wyslij':
        if len(options) < 3:
            ch.send("Musisz podać do kogo chcesz wysłać wiadomość oraz jej temat.")
            return
        tgt = mudsys.get_player(options[1])
        if tgt == None:
            ch.send("Nie ma takiego gracza.")
        elif ch.notepad == '':
            ch.send("Twój notatnik jest obecnie pusty. Spróbuj coś napisać.")
        elif tgt.is_npc:
            ch.send("Nie możesz wysłać wiadomości do postaci nie będącej graczem.")
        elif len(tgt.aux('mail').mails) >= 20:
            ch.send("Nie możesz wysłać wiadomości do " + tgt.name + "$g ponieważ ma już pełną skrzynkę.")
        elif ch.hasPrefs('muted'):
            ch.send("Nie możesz wysyłać wiadomości ponieważ zostałeś uciszony.")
        elif ch.name == tgt.name:
            ch.send("Nie możesz wysłać wiadomości sam sobie.")
        else:
            tgt.aux('mail').mails.append(Mail(ch.name, options[2], ch.notepad, 'N'))
            ch.send("Wysłałeś wiadomość do " + tgt.name + "$g.")
            tgt.send("Dostałeś wiadomość od " + ch.name + "$g.")
            mudsys_do_save(tgt)

################################################################################
# initialization
################################################################################
auxiliary.install("mail", MailData, "character")

mudsys.add_cmd("poczta", None, cmd_mail, "player", False)
