# -*- coding: utf-8 -*-
'''
movement.py

all of the functions concerned with movement and position change
'''
import inform, hooks, mudsys, mud, gainlevel



# a ranking of positions
positions     = ["unconscious", "sleeping", "sitting", "standing", "flying"]
pos_act       = ["upada",   "zasypia",    "siada",     "wstaje",    "leci"]

# stuff for handling movement
dir_name = ["północ", "wschód", "południe", "zachód", "północny-wschód", "północny-zachód",
                 "południowy-zachód", "południowy-wschód", "góra", "dół"]
dir_abbr = ["pn", "w", "pd", "z", "pnw", "pnz", "pdz", "pdw", "g", "d"]
dir_opp  = [  2,   3,   0,   1,   6,     7,    4,    5,   9,   8]



def try_use_furniture(ch, obj, pos):
    '''attempts to resituate a person on the piece of furniture'''
    # are we already on it?
    if ch.on == obj:
        ch.send("Już siedzisz " + ch.on.furniture_type + " " + ch.on.name + ".")
    # make sure we have room
    elif obj.furniture_capacity <= len(obj.chars):
        ch.send("Nie ma już tutaj miejsca aby usiąść.")
    else:
        # are we already on something? get up first
        if ch.on:
            if obj.furniture_type == "na":
                wakeup = "z"
            else:
                wakeup = "od"
            mud.message(ch,None,ch.on,None,True,"to_char",
                        "Wstajesz " + wakeup + " $o.")
            mud.message(ch,None,ch.on,None,True,"to_room",
                        "$n wstaje " + wakeup + " $o.")
            ch.on = None

        # send our mud.messages for sitting down
        act = pos_act[positions.index(pos)]
        mud.message(ch, None, obj, None, True, "to_char", act[0].upper() + act[1:] + "sz " + obj.furniture_type + " $o$$l.")
        mud.message(ch, None, obj, None, True, "to_room", "$n " + act + obj.furniture_type +" $o$$l.")

        # place ourselves down on our new furniture
        ch.on  = obj
        ch.pos = pos
        return True

    # we didn't manage to get on the furniture
    return False

def try_change_pos(ch, pos):
    '''this function attempts to change the position of the person'''
    if ch.pos == pos:
        ch.send("Już " + pos + ".")
        return False
    else:
        if ch.pos == "flying":
            mud.message(ch,None,None,None,True, "to_char", "Lądujesz.")
            mud.message(ch,None,None,None,True, "to_room", "$n ląduje.")

        act = pos_act[positions.index(pos)]
        mud.message(ch, None, None, None, True, "to_char", act[0].upper() + act[1:] + "sz.")
        mud.message(ch, None, None, None, True, "to_room", "$n "  + act + ".")
        ch.pos = pos
        return True

def cmd_sit(ch, cmd, arg):
    '''If standing, attempts to sit on the ground.'''
    try:
        obj, = mud.parse_args(ch, True, cmd, arg, "| [na] obj.room")
    except: return

    if obj == None:
        try_change_pos(ch, "sitting")
    elif obj.istype("furniture"):
        try_use_furniture(ch, obj, "sitting")
    else:
        ch.send("Nie możesz usiąść na " + ch.see_as(obj) + ".")

def cmd_sleep(ch, cmd, arg):
    '''If awake, attempts to lay down and sleep.'''
    try:
        obj, = mud.parse_args(ch, True, cmd, arg, "| [on] obj.room")
    except: return

    if obj == None:
        try_change_pos(ch, "sleeping")
    elif obj.istype("furniture"):
        try_use_furniture(ch, obj, "sleeping")
    else:
        ch.send("Nie możesz spać na " + ch.see_as(obj) + ".")

def cmd_stand(ch, cmd, arg):
    '''If sitting, attempts to stand. If flying, attempts to land.'''
    try_change_pos(ch, "standing")

def cmd_wake(ch, cmd, arg):
    '''If sleep, attempts to wake up and sit.'''
    mud.message(ch,None,None,None,True, "to_char",
                "Budzisz się i siadasz.")
    mud.message(ch,None,None,None,True, "to_room",
                "$n budzi się i siada.")
    ch.pos = "sitting"

def dir_opposite(dir):
    '''returns the opposite direction of the specified one, or None if none.'''
    try:
        return dir_name[dir_opp[dir_index(dir)]]
    except: return None

def dir_index(dir):
    '''returns the index of the direction name'''
    try:
        return dir_name.index(dir)
    except: pass
    try:
        return dir_abbr.index(dir)
    except: pass
    return -1

def try_move_mssg(ch, dir):
    '''Handles all moving of characters from one room to another, through
       commands. Attempts a move. If successful, returns the exit left through.
       Informs people of our moving'''
    ex, success = try_move(ch, dir, True)
    return ex

def try_move(ch, dir, mssg = False):
    '''Handles all moving of characters from one room to another, through
       commands. Attempts a move. If successful, returns the exit left
       through.'''
    ex      = ch.room.exit(dir)
    success = False

    exname = "it"
    if ex != None and ex.name != "":
        exname = ex.name 
    
    # did we find an exit?
    if ex == None or not ch.cansee(ex):
        ch.send("Niestety, nie ma wyjścia w tym kierunku.")
    elif ex.is_closed:
        ch.send("Musisz najpierw otworzyć " + exname + ".")
    elif ex.dest == None:
        ch.send("Nie wygląda na to aby " + exname + " prowadziło dokądkolwiek!")
    else:
        
        #did we have enough strength to move? Only for chars (Unfinished)
        if ch.is_pc:
            if ch.aux('stats').values[4] == 0:
                ch.send("Nie możesz chodzić, kiedy jesteś martwy!")
                return ex, success
            terrains1 = ["Inside", "City", "Road", "Alley", "Bridge", 'Cavern']
            terrains2 = ['Field', 'Plains', 'Meadow', 'Forest', 'Sand']
            terrains3 = ['Deep Forest', 'Hills', 'High Hills', 'Swamp', 'Desert']
            terrains4 = ['Mountains', 'Deep Swamps', 'Ice', 'Glacier']
            fatigue = ch.aux('stats').values[5]
            max_fatigue = ch.aux('stats').secondvalues[5]
            if ch.pos != 'flying':
                if ex.dest.terrain in terrains1:
                    fatigue += 1
                elif ex.dest.terrain in terrains2:
                    fatigue += 2
                elif ex.dest.terrain in terrains3:
                    fatigue += 3
                elif ex.dest.terrain in terrains4:
                    fatigue += 4
            else:
                fatigue += 2
            if ch.hidden > 0:
                fatigue = fatigue * 2
            if fatigue >= max_fatigue:
                ch.send("Jesteś zbyt zmęczony aby podróżować, odpocznij trochę!")
                ch.aux('stats').values[5] = max_fatigue
                return ex, success
            ch.aux('stats').values[5] = fatigue
            ch.aux('stats').secondvalues[3] += 0.01
            gainlevel.check_exp(ch, "con", "stat")

        old_room = ch.room
        dirnum   = dir_index(dir)

        # send out our leave mud.messages as needed. Is anyone in the old room?
        if mssg == True:
            if ch.pos == "flying":
                walk_mssg = "odlatuje"
            else:
                walk_mssg = "odchodzi"
            if ex.leave_mssg != '':
                mud.message(ch, None, None, None, True, "to_room",ex.leave_mssg)
            elif dirnum == -1:
                mud.message(ch, None, None, None, True, "to_room", "$n " + walk_mssg  + ".")
            else:
                mud.message(ch, None, None, None, True, "to_room", "$n " + walk_mssg + " na " + dir_name[dirnum] + ".")

        # run our leave hooks
        hooks.run("exit", hooks.build_info("ch rm ex", (ch, ch.room, ex)))

        # if a hook hasn't moved us, go through with going through the exit
        if ch.room == old_room:
            ch.room = ex.dest

        # stuff that happens before we 'look'
        hooks.run("pre_enter", hooks.build_info("ch rm", (ch, ch.room)))
            
        ch.act("patrz")

        # send out our enter mud.messages as needed
        if mssg == True:
            if ex.enter_mssg != '':
                mud.message(ch,None,None,None,True,"to_room",ex.enter_mssg)
            elif dirnum == -1:
                mud.message(ch,None,None,None,True,"to_room","$n przybywa.")
            else:
                mud.message(ch, None, None, None, True, "to_room",
                            "$n przybywa z " + dir_name[dir_opp[dirnum]]+".")

        # run our enter hooks
        hooks.run("enter", hooks.build_info("ch rm", (ch, ch.room)))
        success = True

    # return the exit we found (if we found any)
    return ex, success

def cmd_move(ch, cmd, arg):
    '''A basic movement command, relocating you to another room in the
       specified direction.'''
    # cmd_move is the basic entry to all of the movement utilities.
    # See try_move() in movement.py
    
    dirnames = ["polnoc", "wschod", "poludnie", "zachod", "polnocnywschod", "polnocnyzachod",
                "poludniowyzachod", "poludniowywschod", "gora", "dol"]

    try:
        index = dirnames.index(cmd)
    except ValueError:
        index = dir_abbr.index(cmd)

    try_move_mssg(ch, dir_name[index])

def cmd_fly(ch, cmd, arg):
    '''Usage: wzlec

    Start flying. Available only for faerie.'''
    ch.pos = "flying"
    ch.send("Wznosisz się nad ziemię.")
    mud.message(ch, None, None, None, True, "to_room", "$n wznosi się nad ziemię.")

def cmd_recall(ch, cmd, arg):
    '''Usage: powrot

    Back to fountain in Matherion. It cause 50% of fatigue.'''
    fatigue = ch.aux('stats').values[5]
    max_fatigue = ch.aux('stats').secondvalues[5]
    need_fatigue = max_fatigue / 2
    if fatigue + need_fatigue > max_fatigue:
        ch.send("Jesteś zbyt zmęczony aby użyć tej komendy!")
        return
    ch.aux('stats').values[5] += need_fatigue
    ch.send("Świat zawirował wokół ciebie. Kolory zmieniły się w tęczę...")
    mud.message(ch, None, None, None, True, "to_room", "$n znika w tęczowym błysku.")
    # run our leave hooks
    fountain, = mud.parse_args(ch, True, '', 'fontanna@matherion', 'room')
    ch.room = fountain
    ch.pos = "standing"
    hooks.run("pre_enter", hooks.build_info("ch rm", (ch, ch.room)))
    ch.act("patrz")
    hooks.run("enter", hooks.build_info("ch rm", (ch, ch.room)))
    mud.message(ch, None, None, None, True, "to_room", "$n pojawia się w tęczowym błysku.")

################################################################################
# mud commands
################################################################################
mudsys.add_cmd("polnoc",    None,  cmd_move, "player", True)
mudsys.add_cmd("zachod",    "z",  cmd_move, "player", True)
mudsys.add_cmd("wschod",    "w",  cmd_move, "player", True)
mudsys.add_cmd("poludnie",  None,  cmd_move, "player", True)
mudsys.add_cmd("gora",      "g",  cmd_move, "player", True)
mudsys.add_cmd("dol",       "d",  cmd_move, "player", True)
mudsys.add_cmd("polnocnyzachod", None, cmd_move, "player", True)
mudsys.add_cmd("polnocnywschod", None, cmd_move, "player", True)
mudsys.add_cmd("poludniowyzachod", None, cmd_move, "player", True)
mudsys.add_cmd("poludniowywschod", None, cmd_move, "player", True)
mudsys.add_cmd("pn",         None, cmd_move, "player", True)
mudsys.add_cmd("pd",         None, cmd_move, "player", True)
mudsys.add_cmd("pnz",        None, cmd_move, "player", True)
mudsys.add_cmd("pnw",        None, cmd_move, "player", True)
mudsys.add_cmd("pdz",        None, cmd_move, "player", True)
mudsys.add_cmd("pdw",        None, cmd_move, "player", True)
mudsys.add_cmd("wzlec",       None, cmd_fly,  "player", True)
mudsys.add_cmd("obudz",     None, cmd_wake, "player", True)
mudsys.add_cmd("spij",      None, cmd_sleep,"player", True)
mudsys.add_cmd("wstan",     None, cmd_stand,"player", True)
mudsys.add_cmd("laduj",     None, cmd_stand,"player", True)
mudsys.add_cmd("usiadz",    None, cmd_sit,  "player", True)
mudsys.add_cmd("powrot",    None, cmd_recall, "player", True)

# The mud needs to know our command for movement as well
mudsys.set_cmd_move(cmd_move)

# useful mud methods
mud.dir_opposite = dir_opposite

def chk_can_move(ch, cmd):
    if not ch.pos in ["standing", "flying"]:
        if ch.pos == "sitting":
            pozycja = "siedzisz"
        elif ch.pos == "sleeping":
            pozycja = "śpisz"
        elif ch.pos == "unconscious":
            pozycja = "jesteś nieprzytomny"
        ch.send("Nie możesz poruszać się kiedy " + pozycja + "!")
        return False

for cmd in ["polnoc", "zachod", "wschod", "poludnie", "gora", "dol", "polnocnyzachod", "polnocnywschod", "poludniowyzachod", "poludniowywschod", "pnz", "pnw", "pdz", "pdw", "pn", "pd"]:
    mudsys.register_dflt_move_cmd(cmd)
mudsys.register_move_check(chk_can_move)

def chk_wake(ch, cmd):
    if not ch.pos == "sleeping":
        ch.send("Musisz spać aby móc się obudzić.")
        return False
mudsys.add_cmd_check("obudz", chk_wake)

def chk_sleep(ch, cmd):
    if ch.pos == "sleeping":
        ch.send("Już śpisz!")
        return False
    elif ch.pos == "unconscious":
        ch.send("Nie możesz spać kiedy jesteś nieprzytomny.")
        return False
mudsys.add_cmd_check("spij", chk_sleep)

def chk_stand(ch, cmd):
    if ch.pos == "standing":
        ch.send("Już stoisz!")
        return False
    elif ch.pos != "sitting":
        ch.send("Nie możesz wstać jeżeli nie siedzisz.")
        return False
mudsys.add_cmd_check("wstan", chk_stand)

def chk_land(ch, cmd):
    if ch.pos == "standing":
        ch.send("Jesteś już na ziemi!")
        return False
    elif ch.pos != "flying":
        ch.send("Nie możesz wylądować jeżeli nie latasz.")
        return False
mudsys.add_cmd_check("laduj", chk_land)

def chk_sit(ch, cmd):
    if ch.pos == "sitting":
        ch.send("Już siedzisz!")
        return False
    elif ch.pos != "standing":
        ch.send("Musisz stać aby móc usiąść.")
        return False
mudsys.add_cmd_check("usiadz", chk_sit)

def chk_fly(ch, cmd):
    if ch.pos == "flying":
        ch.send("Już latasz!")
        return False
    elif ch.pos != "standing":
        ch.send("Nie możesz wznieść się, jeżeli nie stoisz!")
        return False
    if ch.race != "faerie":
        ch.send("Tylko rasa faerie może latać!")
        return False
mudsys.add_cmd_check("wzlec", chk_fly)

def chk_can_recall(ch, cmd):
    if not ch.pos in ["standing", "flying"]:
        if ch.pos == "sitting":
            pozycja = "siedzisz"
        elif ch.pos == "sleeping":
            pozycja = "śpisz"
        elif ch.pos == "unconscious":
            pozycja = "jesteś nieprzytomny"
        ch.send("Nie możesz poruszać się kiedy " + pozycja + "!")
        return False
    if ch.hasvar('enemy'):
        ch.send("Najpierw musisz zakończyć walkę!")
        return False
mudsys.add_cmd_check("powrot", chk_can_recall)
