# -*- coding: utf-8 -*-
'''
odmiana.py

Dostarcza strukrurę danych oraz funkcje do odmiany nazw i imion w języku polskim
'''

import mudsys, mud, auxiliary, storage, olc

################################################################################
# definicja typu odmiana
################################################################################

class OdmianaData:
    '''Struktura danych dla odmiany.'''

    def __init__(self, set = None):
        '''Standardowy konstruktor klasy.'''
        self.przypadki = []

        if set != None and len(set.readString('przypadki')) > 0:
            self.przypadki = set.readString('przypadki').split(';')

    def copyTo(self, to):
        to.przypadki = self.przypadki

    def copy(self):
        newData = OdmianaData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString("przypadki", ';'.join(self.przypadki))
        return set

################################################################################
# odmiana OLC
################################################################################

def odmolc_menu(sock, data):
    '''Wyświetl wszystkie przypadki i obecne wartości jakie posiadają.'''
    buf = ['Dopełniacz (kogo,czego nie ma):', 'Celownik (komu, czemu przyglądam się):', 'Biernik (kogo, co widzę):', 'Narzędnik (z kim, czym idę):', 'Miejscownik (o kim, czym mówię):', 'Wołacz (witaj!):']
    buf2 = []
    i = 0
    if len(data.aux('odmiana').przypadki) > 0:
        for opcja in buf:
            buf2.append("%d) %s {c%s{g" % ((i + 1), opcja, data.aux('odmiana').przypadki[i]))
            i += 1
    else:
        for opcja in buf:
            buf2.append("%d) %s {c%s{g" % ((i + 1), opcja, data.name))
            i += 1
    sock.send("\r\n".join(buf2))

def odmolc_chooser(sock, data, option):
    '''Wybieranie przypadku do zmiany.'''
    try:
        opt = int(option) - 1
        if opt < 0 or opt > 5:
            return olc.MENU_CHOICE_INVALID
        sock.send("Wprowadź nową wartość: ", newline = False)
        return opt + 1
    except:
        return olc.MENU_CHOICE_INVALID

def odmolc_parser(sock, owner, choice, arg):
    '''Ustawianie nowej wartości dla przypadku'''
    try:
        opt = choice - 1
        #sprawdźmy czy poprzednie wartości są ustawione, jeżeli nie, wypełnijmy je
        if len(owner.aux('odmiana').przypadki) == 0:
            for i in range(0, 6):
                owner.aux('odmiana').przypadki.append(owner.name)
        owner.aux('odmiana').przypadki[opt] = arg
        return True
    except:
        return False

def odm_to_proto(owner):
    '''Zapisanie odmiany do pliku.'''
    if len(owner.aux('odmiana').przypadki) > 0:
        przypadki = owner.aux('odmiana').przypadki
    else:
        przypadki = []
        for i in range(0, 6):
            przypadki.append(owner.name)
    odmiana = '['
    for i in range(0, 6):
        odmiana += "'" + przypadki[i] + "'"
        if i < 5:
            odmiana += ', '
    odmiana += ']'
    return 'me.aux("odmiana").przypadki = %s\n' % odmiana

def odmolc_extend_menu(sock, data):
    '''Wyświelt opcję w głównym menu'''
    sock.send("Odmiana imienia/nazwy")

def odmolc_choice_exec(sock, data):
    '''Edit odmianę'''
    olc.do_olc(sock, odmolc_menu, odmolc_chooser, odmolc_parser, None, data)
    return olc.MENU_NOCHOICE

################################################################################
# funkcje
################################################################################

def zastap_odmiane(text, sender):
    '''Zamienia zmienne na właściwą odmianę.'''
    zmienne = ['$g', '$d', '$a', '$i', '$l', '$v']
    i = 0
    for zmienna in zmienne:
        index = 0
        while True:
            index = text.find(zmienna)
            if index == -1:
                break
            spindex = text.rfind(' ', 0, index)
            if spindex == -1:
                spindex = 0
            else:
                spindex += 1
            name = text[spindex:index]
            if name != sender.name:
                try:
                    owner, type = mud.parse_args(sender, True, '', name, '{ ch.world obj.world }')
                except:
                    return text
            else:
                owner = sender
            if len(owner.aux('odmiana').przypadki) > 0:
                text = text.replace(owner.name + zmienna, owner.aux('odmiana').przypadki[i])
            else:
                text = text.replace(owner.name + zmienna, owner.name)
        i += 1
    return text
                
################################################################################
# komendy
################################################################################

def cmd_odmiana(ch, cmd, arg):
    '''Użycie: odmiana <nr przypadku> <wartość>

       Wpisanie samej komendy, pokaże aktualne wartości dla odmiany twojego imienia.
       Aby zmienić jakąś wartość, do komendy dodaj numer przypadku i nową wartość.
       Np odmiana 1 Thindila.'''
    args = arg.split()
    buf = ['Dopełniacz (kogo,czego nie ma)', 'Celownik (komu, czemu przyglądam się)', 'Biernik (kogo, co widzę)', 'Narzędnik (z kim, czym idę)', 'Miejscownik (o kim, czym mówię)', 'Wołacz (witaj!)']
    #pokaż obecne ustawienia
    if len(args) == 0:
        buf2 = []
        i = 0
        if len(ch.aux('odmiana').przypadki) > 0:
            for opcja in buf:
                buf2.append("%d) %s: {c%s{n" % ((i + 1), opcja, ch.aux('odmiana').przypadki[i]))
                i += 1
        else:
            for opcja in buf:
                buf2.append("%d) %s: {c%s{n" % ((i + 1), opcja, ch.name))
                i += 1
        ch.send("\r\n".join(buf2))
    else:
        if not args[0].isdigit():
            ch.send("Podaj numer przypadku")
            return
        index = int(args[0]) - 1
        if index < 0 or index > 5:
            ch.send("Numer musi być wartością od 1 do 6")
            return
        if len(ch.aux('odmiana').przypadki) > 0:
            ch.aux('odmiana').przypadki[index] = ' '.join(args[1:])
        else:
            for i in range(0, 6):
                if i == index:
                    ch.aux('odmiana').przypadki.append(' '.join(args[1:]))
                else:
                    ch.aux('odmiana').przypadki.append(ch.name)
        ch.send("Zmieniłeś odmianę dla przypadku %s na %s" % (buf[index], ' '.join(args[1:])))

def cmd_sodmiana(ch, cmd, arg):
    '''Użycie sodmiana <postać lub obiekt>

       Sprawdź odmianę dla wybranej postaci bądź obiektu - administracyjna komenda'''

    try:
        data, = mud.parse_args(ch, False, cmd, arg, "ch.world")
    except:
        try:
            data, = mud.parse_args(ch, False, cmd, arg, "obj.world")
        except:
            ch.send("Nie znaleziono postaci ani obiektu o tej nazwie.")

    buf = ['Dopełniacz (kogo,czego nie ma):', 'Celownik (komu, czemu przyglądam się):', 'Biernik (kogo, co widzę):', 'Narzędnik (z kim, czym idę):', 'Miejscownik (o kim, czym mówię):', 'Wołacz (witaj!):']
    buf2 = []
    i = 0
    if len(data.aux('odmiana').przypadki) > 0:
        for opcja in buf:
            buf2.append("%d) %s {c%s{n" % ((i + 1), opcja, data.aux('odmiana').przypadki[i]))
            i += 1
    else:
        for opcja in buf:
            buf2.append("%d) %s {c%s{n" % ((i + 1), opcja, data.name))
            i += 1
    ch.send("\r\n".join(buf2))

################################################################################
# inizjalizacja
################################################################################
auxiliary.install("odmiana", OdmianaData, "character, object")

olc.extend("medit", "O", odmolc_extend_menu, odmolc_choice_exec, None, None,
           odm_to_proto)
olc.extend("oedit", "O", odmolc_extend_menu, odmolc_choice_exec, None, None,
           odm_to_proto)
olc.extend("pcedit", "O", odmolc_extend_menu, odmolc_choice_exec, None, None, None)

mudsys.add_cmd("sodmiana",        None, cmd_sodmiana,         "wizard",  False)
mudsys.add_cmd("odmiana",         None, cmd_odmiana,          "player",  False)
