# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
prompt.py

Show prompt to user with informations about character.
At this moment show only health and fatigue status.
'''

import mudsys, mud, socket, char, hooks, odmiana

################################################################################
# Functions
################################################################################

def health_status(ch):
    '''function check health status of player and return
       text to show.'''
    curr_hp = ch.aux('stats').values[4]
    if curr_hp == 0:
        return "{WMARTWY{n"
    max_hp = ch.aux('stats').secondvalues[4]
    if curr_hp == max_hp:
        return "{GPełnia zdrowia{n"
    perc_hp = (float(curr_hp) / max_hp) * 100
    health_text = ""
    if perc_hp > 89 and perc_hp < 100:
        health_text = "{gKilka zadrapiań{n"
    elif perc_hp > 79 and perc_hp < 90:
        health_text = "{BKilka siniaków{n"
    elif perc_hp > 69 and perc_hp < 80:
        health_text = "{bLekko poobijany{n"
    elif perc_hp > 59 and perc_hp < 70:
        health_text = "{CMocno poobijany{n"
    elif perc_hp > 49 and perc_hp < 60:
        health_text = "{cPoturbowany{n"
    elif perc_hp > 39 and perc_hp < 50:
        health_text = "{YLekko ranny{n"
    elif perc_hp > 29 and perc_hp < 40:
        health_text = "{yRanny{n"
    elif perc_hp > 19 and perc_hp < 30:
        health_text = "{RCiężko ranny{n"
    elif perc_hp > 9 and perc_hp < 20:
        health_text = "{rLedwo żywy{n"
    elif perc_hp > 0 and perc_hp < 10:
        health_text = "{wUMIERAJĄCY{n"
    return health_text

def fatigue_status(ch):
    '''function check fatigue status of player and return
       text to show.'''
    curr_fatigue = ch.aux('stats').values[5]
    if curr_fatigue == 0:
        return "{GWypoczęty{n"
    max_fatigue = ch.aux('stats').secondvalues[5]
    if curr_fatigue == max_fatigue:
        return "{WNIEPRZYTOMNY{n"
    perc_fatigue = (float(curr_fatigue) / max_fatigue) * 100
    fatigue_text = ""
    if perc_fatigue > 0 and perc_fatigue < 10:
        fatigue_text = "{gRześki{n"
    elif perc_fatigue > 9 and perc_fatigue < 20:
        fatigue_text = "{BWytrzymały{n"
    elif perc_fatigue > 19 and perc_fatigue < 30:
        fatigue_text = "{bNieco zmęczony{n"
    elif perc_fatigue > 29 and perc_fatigue < 40:
        fatigue_text = "{CTrochę zmęczony{n"
    elif perc_fatigue > 39 and perc_fatigue < 50:
        fatigue_text = "{cZmęczony{n"
    elif perc_fatigue > 49 and perc_fatigue < 60:
        fatigue_text = "{YBardzo zmęczony{n"
    elif perc_fatigue > 59 and perc_fatigue < 70:
        fatigue_text = "{yPadnięty{n"
    elif perc_fatigue > 69 and perc_fatigue < 80:
        fatigue_text = "{RZataczający się{n"
    elif perc_fatigue > 79 and perc_fatigue < 90:
        fatigue_text = "{rOmdlewający{n"
    elif perc_fatigue > 89 and perc_fatigue < 100:
        fatigue_text = "{wSŁANIAJĄCY SIĘ{n"
    return fatigue_text

def encode_text(text, encoding):
    '''Encode selected text.'''
    if encoding == 'brak':
        oldchars = ['ą', 'ć', 'ę', 'ó', 'ś', 'ł', 'ż', 'ź', 'ń']
        newchars = ['a', 'c', 'e', 'o', 's', 'l', 'z', 'z', 'n']
        i = 0
        for char in oldchars:
            text = text.replace(char, newchars[i])
            i += 1
    else:
        text = text.decode('utf-8')
        text = text.encode(encoding)
    return text

def new_prompt(sock):
    '''Function override default prompt.'''
    if sock.state != 'playing':
        return
    sock.send_raw("\r\n[" + health_status(sock.ch) + "] [" + fatigue_status(sock.ch) + "]")

################################################################################
# Hooks
################################################################################
def show_prompt_hook(info):
    '''Change text encoding in prompt if needed.'''
    sock, = hooks.parse_info(info)
    if sock.account == None:
        return
    if sock.aux('misc').encoding != 'utf-8':
        sock.outbound_text = encode_text(sock.outbound_text, sock.aux('misc').encoding)

def encoding_hook(info):
    '''Change text encoding if needed.'''
    sock, = hooks.parse_info(info)
    if sock.account == None:
        return
    if sock.ch != None:
        sock.outbound_text = odmiana.zastap_odmiane(sock.outbound_text, sock.ch)
    if sock.aux('misc').encoding != 'utf-8':
        sock.outbound_text = encode_text(sock.outbound_text, sock.aux('misc').encoding)

def inbound_hook(info):
    '''Change text encoding of incoming text if needed.'''
    sock, = hooks.parse_info(info)
    if sock.account == None:
        return
    if sock.aux('misc').encoding != 'utf-8':
        text = sock.inbound_text
        if sock.aux('misc').encoding != 'brak':
            text = text.decode(sock.aux('misc').encoding)
            text = text.encode('utf-8')
        sock.inbound_text = text

################################################################################
# initializing and unloading our hooks
################################################################################
hooks.add("process_outbound_prompt", show_prompt_hook)
hooks.add("process_outbound_text", encoding_hook)
hooks.add("receive_text", inbound_hook)
mudsys.show_prompt = new_prompt

def __unload__():
    '''deatach module from server'''
    hooks.remove("process_outbound_prompt", show_prompt_hook)
    hooks.remove("process_outboud_text", encoding_hook)
    hooks.remove("receive_text", inbound_hook)
