# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
rest.py

Recovering health points and fatigue during time
'''

import mudsys, mud, char, hooks, event, math

################################################################################
# Events
################################################################################
def resting_event(owner, void, info):
    '''Function recover health points and fatigure over time'''
    if owner == None:
        return
    #count restore multiplier - base on player position, thirst and hunger
    if owner.position == "standing" or owner.position == "flying":
        multiplier = 0.1
    elif owner.position == "sitting":
        multiplier = 0.15
    elif owner.position == "sleeping" or owner.position == "unconscious":
        multiplier = 0.2
    if owner.is_pc:
        condition_mod = float(owner.aux('stats').values[6]) / float(owner.aux('stats').secondvalues[6])
        multiplier -= (multiplier * condition_mod)
        condition_mod = float(owner.aux('stats').values[7]) / float(owner.aux('stats').secondvalues[7])
        multiplier -= (multiplier * condition_mod)
    #raise hit points
    cur_val = owner.aux('stats').values[4]
    max_val = owner.aux('stats').secondvalues[4]
    recover = math.ceil(float(max_val) * multiplier)
    if recover < 1:
        recover = 1
    cur_val = cur_val + recover
    if owner.position == "unconscious":
        owner.position = "sitting"
    if cur_val > max_val:
        cur_val = max_val
    owner.aux('stats').values[4] = int(cur_val)
    #lower fatigue for players only
    if owner.is_pc:
        cur_val = owner.aux('stats').values[5]
        max_val = owner.aux('stats').secondvalues[5]
        recover = math.ceil(float(max_val) * multiplier)
        if recover < 1:
            recover = 1
        cur_val = cur_val - recover
        if cur_val < 0:
            cur_val = 0
        owner.aux('stats').values[5] = int(cur_val)
    event.start_event(owner, 70, resting_event, None, '')

def condition_event(owner, void, info):
    '''Event raise hunger and thirst of character'''
    if owner == None:
        return
    #raise hunger
    cur_val = owner.aux('stats').values[6]
    max_val = owner.aux('stats').secondvalues[6]
    if cur_val < max_val:
        cur_val = cur_val + 1
    owner.aux('stats').values[6] = int(cur_val)
    percent = (float(cur_val) / float(max_val)) * 100
    if percent > 74:
        owner.send("Jesteś głodny!")
    #raise thirst
    cur_val = owner.aux('stats').values[7]
    max_val = owner.aux('stats').secondvalues[7]
    if cur_val < max_val:
        cur_val = cur_val + 1
    owner.aux('stats').values[7] = int(cur_val)
    percent = (float(cur_val) / float(max_val)) * 100
    if percent > 74:
        owner.send("Jesteś spragniony!")
    #start event again
    event.start_event(owner, 180, condition_event, None, '')

################################################################################
# hooks
################################################################################
def resting_hook(info):
    '''start resting over time'''
    ch, = hooks.parse_info(info)
    if ch == None or not ch.is_pc:
        return
    event.start_event(ch, 70, resting_event, None, '')

def condition_hook(info):
    '''raise hunger and thirst over time'''
    ch, = hooks.parse_info(info)
    if ch == None or not ch.is_pc:
        return
    event.start_event(ch, 180, condition_event, None, '')

################################################################################
# initialization and unloading
################################################################################
hooks.add("char_to_game", resting_hook)
hooks.add("char_to_game", condition_hook)

def __unload__():
    '''things that need to be detached when the module is un/reloaded'''
    hooks.remove("char_to_game", resting_hook)
    hooks.remove("char_to_game", condition_hook)
