# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
skills.py

Provide data and functions for handle players skills
'''

import mudsys, mud, auxiliary, storage, olc, mudsock

################################################################################
# skils type definition
################################################################################

class SkillsData:
    '''Data for skills.'''

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.names = []
        self.levels = []
        self.exp = []

        if set != None and len(set.readString("names")) > 0:
            self.names = set.readString("names").split(';')
            if len(self.names) == 0:
                return
            levels = set.readString("levels").split(';')
            for level in levels:
                self.levels.append(int(level))
            experience = set.readString("exp").split(';')
            for value in experience:
                self.exp.append(float(value))

    def copyTo(self, to):
        to.names = self.names
        to.levels = self.levels
        to.exp = self.exp

    def copy(self):
        newData = SkillsData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString("names", ';'.join(self.names))
        values = []
        for value in self.levels:
            values.append(str(value))
        experience = []
        for value in self.exp:
            experience.append(str(value))
        set.storeString("levels", ';'.join(values))
        set.storeString("exp", ';'.join(experience))
        return set

################################################################################
# Skills OLC
################################################################################
SKILL_NEW = 1
SKILL_EDIT = 2
SKILL_DELETE = 3

def skillsolc_menu(sock, data):
    '''Display all skills and values for they.'''
    buf = []
    if len(data.aux('skills').names) == 0:
        buf.append('Brak umiejętności.')
    else:
        for i in range(len(data.aux('skills').names)):
            buf.append("%s: %d" % (data.aux('skills').names[i], data.aux('skills').levels[i]))
    buf.append('')
    buf.append('')
    buf.append('N) Nowa umiejętność')
    if len(data.aux('skills').names) > 0:
        buf.append('E) Edytuj umiejętność')
        buf.append('D) Skasuj umiejętność')
    sock.send('\r\n'.join(buf))

def skillsolc_chooser(sock, data, option):
    '''Try pick skill to set.'''
    options = ['n', 'e', 'd']
    option = option.lower()
    if not option in options:
        return olc.MENU_CHOICE_INVALID
    if option == 'n':
        sock.send("Nazwa nowej umiejętności: ")
        return SKILL_NEW
    else:
        if len(data.aux('skills').names) == 0:
            return olc.MENU_CHOICE_INVALID
        else:
            if option == 'd':
                sock.send("Nazwa umiejętności do skasowania: ")
                return SKILL_DELETE
            else:
                sock.send("Nazwa umiejętności do edycji: ")
                return SKILL_EDIT

def skillsolc_parser(sock, mobile, choice, arg):
    '''Set new skill or level for it.'''
    if choice == SKILL_EDIT or choice == SKILL_DELETE:
        if arg not in mobile.aux('skills').names:
            sock.send('Ta postać nie posiada takiej umiejętności.')
            return False
        i = mobile.aux('skills').names.index(arg)
    if choice == SKILL_DELETE:
        del mobile.aux('skills').names[i]
        del mobile.aux('skills').levels[i]
    elif choice == SKILL_EDIT:
        olc.do_olc(sock, skillolc_menu, skillolc_chooser, None, None, [mobile, i])
    elif choice == SKILL_NEW:
        mobile.aux('skills').names.append(arg)
        mobile.aux('skills').levels.append(0)
        i = mobile.aux('skills').names.index(arg)
        olc.do_olc(sock, skillolc_menu, skillolc_chooser, None, None, [mobile, i])
    return True

def skillsolc_to_proto(thing):
    '''Save skills.'''
    names = '['
    length = len(thing.aux('skills').names)
    if length > 0:
        for i in range(0, length):
            names += "'" + thing.aux('skills').names[i] + "'"
            if i < length - 1:
                names += ', '
    names += ']'
    levels = '['
    if length > 0:
        for i in range(0, length):
            levels += str(thing.aux('skills').levels[i])
            if i < length - 1:
                levels += ', '
    levels += ']'
    buf = 'me.aux("skills").names = %s\n' % names
    buf += 'me.aux("skills").levels = %s\n' % levels
    return buf

def skillsolc_extend_menu(sock, data):
    '''Display option in main menu.'''
    sock.send("Edytuj umiejętności")

def skillsolc_choice_exec(sock, data):
    '''Edit skills.'''
    olc.do_olc(sock, skillsolc_menu, skillsolc_chooser, skillsolc_parser, None, data)
    return olc.MENU_NOCHOICE

def skillolc_menu(sock, data):
    '''Show level of skill.'''
    buf = ['Umiejętność: ' + data[0].aux('skills').names[data[1]]]
    buf.append('Poziom: %d' % data[0].aux('skills').levels[data[1]])
    buf.append('Nowy poziom umiejętności: ')
    sock.send('\r\n'.join(buf))

def skillolc_chooser(sock, data, option):
    '''Set new level for selected skill.'''
    try:
        opt = int(option)
        if opt < 0:
            return olc.MENU_CHOICE_INVALID
        data[0].aux('skills').levels[data[1]] = opt
        return olc.MENU_NOCHOICE
    except:
        return olc.MENU_CHOICE_INVALID

################################################################################
# initialization
################################################################################
auxiliary.install("skills", SkillsData, "character")

mudsys.create_bit('char_prfs', 'immortal')

olc.extend("medit", "U", skillsolc_extend_menu, skillsolc_choice_exec, None, None,
           skillsolc_to_proto)
olc.extend("pcedit", "S", skillsolc_extend_menu, skillsolc_choice_exec, None, None, None)
