# -*- coding: utf-8 -*-
'''
stats.py

Provide data and functions for handle players/mobiles stats
'''

import olc, mudsys, mud, mudsock, auxiliary, storage

################################################################################
# stats type definition
################################################################################

class StatsData:
    '''Data for stats.'''

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.names = ['Siła', 'Zręczność', 'Inteligencja', 'Kondycja', 'Punkty Życia', 'Zmęczenie', 'Głód', 'Pragnienie']
        self.values = [5, 5, 5, 5, 10, 0, 0, 0]
        self.secondvalues = [0.0, 0.0, 0.0, 0.0, 10, 100, 25, 25]

        if set != None and len(set.readString("names")) > 0:
            self.names = set.readString("names").split(';')
            values = set.readString("values").split(';')
            self.values = []
            for value in values:
                self.values.append(int(value))
            secondvalues = set.readString("secondvalues").split(';')
            self.secondvalues = []
            for value in secondvalues:
                self.secondvalues.append(float(value))

    def copyTo(self, to):
        to.names = self.names
        to.values = self.values
        to.secondvalues = self.secondvalues

    def copy(self):
        newData = StatsData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString("names", ';'.join(self.names))
        values = []
        for value in self.values:
            values.append(str(value))
        secondvalues = []
        for value in self.secondvalues:
            secondvalues.append(str(value))
        set.storeString("values", ';'.join(values))
        set.storeString("secondvalues", ';'.join(secondvalues))
        return set

################################################################################
# Stats OLC
################################################################################

def statsolc_menu(sock, data):
    '''Display all stats and values for they.'''
    buf = []
    if data.is_pc:
        amount = len(data.aux('stats').names)
    else:
        amount = 5
    for i in range(amount):
        buf.append("%d) %s: %d" % ((i + 1), data.aux("stats").names[i], data.aux("stats").values[i]))
    sock.send("\r\n".join(buf))

def statsolc_chooser(sock, data, option):
    '''Try to pick stat to set.'''
    try:
        opt = int(option) - 1
        if data.is_pc:
            amount = len(data.aux('stats').names) - 1
        else:
            amount = 4
        if opt < 0 or opt > amount:
            return olc.MENU_CHOICE_INVALID
        sock.send("Wprowadź nowy poziom: ", newline = False)
        return opt + 1
    except:
        return olc.MENU_CHOICE_INVALID

def statsolc_parser(sock, mobile, choice, arg):
    '''Set new level for stat.'''
    try:
        val = int(arg)
        if val < 0:
            return False
        opt = choice - 1
        mobile.aux("stats").values[opt] = val
        return True
    except:
        return False

def stats_to_proto(thing):
    names = "['Siła', 'Zręczność', 'Inteligencja', 'Kondycja', 'Punkty Życia']"
    values = '['
    for i in range(0, 5):
        values += str(thing.aux("stats").values[i])
        if i < 4:
            values += ', '
    values += ']'
    buf = 'me.aux("stats").names = %s\n' % names
    buf += 'me.aux("stats").values = %s\n' % values
    return buf

def statsolc_extend_menu(sock, data):
    '''Display option in main menu'''
    sock.send("Edytuj statystyki")

def statsolc_choice_exec(sock, data):
    '''Edit stats'''
    olc.do_olc(sock, statsolc_menu, statsolc_chooser, statsolc_parser, None, data)
    return olc.MENU_NOCHOICE

################################################################################
# initialization
################################################################################
auxiliary.install("stats", StatsData, "character")

olc.extend("medit", "A", statsolc_extend_menu, statsolc_choice_exec, None, None,
           stats_to_proto)
olc.extend("pcedit", "A", statsolc_extend_menu, statsolc_choice_exec, None, None, None)
