# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010 Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
traps.py

Traps item type.
'''
import olc, mudsys, auxiliary, mudsock, storage, mud, hooks, utils, damage, gainlevel

################################################################################
# item type definition
################################################################################

traps_types = {}

class TrapData:
    '''Data for traps.'''

    __item_type__ = 'trap'

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.type = 'strzałki'
        self.disarm = 0
        self.damage = 1
        self.damagetype = 'impaling'
        self.message = 'wylatują strzałki'

        if set != None:
            self.type = set.readString('type')
            self.disarm = set.readInt('disarm')
            self.damage = set.readInt('damage')
            self.damagetype = set.readString('damagetype')
            self.message = set.readString('message')

    def copyTo(self, to):
        to.type = self.type
        to.disarm = self.disarm
        to.damage = self.damage
        to.damagetype = self.damagetype
        to.message = self.message

    def copy(self):
        newData = TrapData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString('type', self.type)
        set.storeInt('disarm', self.disarm)
        set.storeInt('damage', self.damage)
        set.storeString('damagetype', self.damagetype)
        set.storeString('message', self.message)
        return set

################################################################################
# Traps OLC
################################################################################
__TRAPOLC_TYPE__    = 1
__TRAPOLC_DISARM__  = 2
__TRAPOLC_DAMAGE__  = 3

trapolc_opt_map = {
    '1' : (__TRAPOLC_TYPE__, 'Wybierz typ: '),
    '2' : (__TRAPOLC_DISARM__, 'Wprowadź trudność rozbrojenia: '),
    '3' : (__TRAPOLC_DAMAGE__, 'Wprowadź ilość zadawanych obrażeń: '),
    }

def trapolc_menu(sock, data):
    '''display our menu interface'''
    buf = []
    buf.append('{g1)Typ                       : {c' + data.type)
    buf.append('{g2)Trudność rozbrojenia      : {c' + str(data.disarm))
    buf.append('{g3)Zadawane obrażenia        : {c' + str(data.damage))
    buf.append('{gRodzaj zadawanych obrażeń   : {c' + data.damtype)
    buf.append('{gRodzaj zdarzenia            : {c' + data.message)
    sock.send('\r\n'.join(buf))

def trapolc_chooser(sock, data, option):
    '''figure out what we want to edit, and display a prompt.'''
    if not trapolc_opt_map.has_key(option):
        return olc.MENU_CHOICE_INVALID
    ret, mssg = trapolc_opt_map[option]

    #list our types option
    if ret == __TRAPOLC_TYPE__:
        utils.olc_display_table(sock, traps_types.keys(), 4)
    sock.send_raw(mssg)

def trapolc_parser(sock, data, choice, arg):
    '''parse out what we want to change a specified field to'''
    try:
        arg = int(arg)
    except:
        return

    if choice == __TRAPOLC_TYPE__:
        opts = traps_types.keys()
        arg = int(arg)
        if arg < 0 or arg >= len(opts):
            return False

        newtype = opts[arg]
        if newtype != data.type:
            data.type = newtype
            data.damagetype = traps_types[newtype][0]
            data.message = traps_types[newtype][1]
    elif choice == __TRAPOLC_DISARM__:
        data.disarm = max(int(arg), 1)
    elif choice == __TRAPOLC_DAMAGE__:
        data.damage = max(int(arg), 1)
    else:
        return False
    return True

def trapolc_to_proto(data):
    '''return a script that will generate relevant info to create this item'''
    buf = 'me.get_type_data("trap").type = "' + data.type + '"\n'
    buf += 'me.get_type_data("trap").disarm = ' + str(data.disarm) + '\n'
    buf += 'me.get_type_data("trap").damage = ' + str(data.damage) + '\n'
    buf += 'me.get_type_data("trap").damagetype = "' + data.damagetype + '"\n'
    buf += 'me.get_type_data("trap").message = "' + data.message + '"\n'
    return buf

################################################################################
# game hooks
################################################################################

def trap_fire_hook(info):
    '''fire trap and made damage to everyone in room, when someone left room.'''
    ch, room, ex = hooks.parse_info(info)

    #did in room is trap?
    trap = None
    for item in room.objs:
        if item.istype('trap'):
            trap = item
            break
    #no point in continuing
    if trap == None:
        return
    #damage all characters in room
    amount = len(room.chars)
    dmg = max((trap.get_type_data('trap').damage / amount), 1)
    damagetype = trap.get_type_data('trap').damagetype
    for char in room.chars:
        #count damage
        chArmor, chArmorHit, chBody = damage.get_armor(char)
        dmg, hitpart, hitArmor = damage.count_damage(chBody, chArmor, dmg, damagetype)
        if char.is_pc:
            char.send('Z ' + trap.name + '$g ' + trap.get_type_data('message') + ' i trafiają ciebie w ' + hitpart + '!')
            if hitArmor:
                chArmorHit[hitpart] = True
        if dmg > 0:
            char.aux('stats').values[4] -= dmg
            if char.aux('stats').values[4] < 0:
                char.aux('stats').values[4] = 0
        damage.check_armor(char, chArmorHit)
        if char.aux('stats').values[4] == 0:
            damage.create_corpse(char, trap, 'object')
    #remove trap
    mud.extract(trap)

################################################################################
# game commands
################################################################################

def cmd_disarm(ch, cmd, arg):
    '''Disarm trap.'''
    try:
        trap, = mud.parse_args(ch, True, cmd, arg, "obj.room.inv")
    except:
        return

    if not trap.istype('trap'):
        ch.send('To nie jest pułapka.')
        return
    if 'rozbrajać pułapki' in ch.aux('cooldowns').names:
        ch.send('Przed chwilą już rozbrajałeś pułapkę, poczekaj trochę zanim zrobisz to ponownie.')
        return
    #count chance to disarm trap
    chroll = ch.aux('stats').values[1] + random.randint(0, 99)
    hasSkill = -1
    if 'Rozbrajanie pułapek' in ch.aux('skills').names:
        hasSkill = ch.aux('skills').names.index('Rozbrajanie pułapek')
        chroll += ch.aux('skills').levels[hasSkill]
    if chroll >= trap.get_type_data('trap').disarm:
        if hasSkill > -1:
            ch.aux('skills').exp[hasSkill] += chance
        else:
            ch.aux('skills').names.append('Rozbrajanie pułapek')
            ch.aux('skills').levels.append(0)
            ch.aux('skills').exp.append(chance)
        gainlevel.check_exp(ch, 'Rozbrajanie pułapek', 'skill')
        ch.aux('stats').secondvalues[1] += chance
        gainlevel.check_exp(ch, 'dex', 'stat')
        ch.send('Udało Ci się rozbroić pułapkę.')
        mud.extract(trap)
    else:
        if hasSkill > -1:
            ch.aux('skills').exp[hasSkill] += 1
            gainlevel.check_exp(ch, 'Rozbrajanie pułapek', 'skill')
        else:
            ch.aux('skills').names.append('Rozbrajanie pułapek')
            ch.aux('skills').levels.append(0)
            ch.aux('skills').exp.append(1)
        ch.aux('stats').secondvalues[1] += 1
        gainlevel.check_exp(ch, 'dex', 'stat')
        ch.send('Nie udało Ci się rozbroić pułapki.')
    ch.aux('cooldowns').names.append('rozbrajać pułapki')
    ch.aux('cooldowns').times.append(10)

################################################################################
# initialization
################################################################################

#initialize our item data, and its OLC
mudsys.item_add_type(TrapData.__item_type__, TrapData)
olc.item_add_olc(TrapData.__item_type__,
                 trapolc_menu, trapolc_chooser, trapolc_parser,
                 None, trapolc_to_proto)

#add hooks for traps
hooks.add('exit', trap_fire_hook)

#add commands
mudsys.add_cmd("rozbroj", None, cmd_disarm, "player", True)

#initialize some default trap types
traps_types['strzałki'] = ['impaling', 'wylatują strzałki']
traps_types['kamienie'] = ['crush', 'spadają kamienie']
traps_types['ostrza'] = ['cut', 'wyskakują ostrza']

def __unload__():
    '''remove all hooks.'''
    hooks.remove('exit', trap_fire_hook)
