# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
weather.py

Provide data structure and functions for weather system.
'''

import mudsys, mud, auxiliary, storage, olc, mudsock, utils, hooks, zone, os, random, char

################################################################################
# weather data type definition
################################################################################

weather_climate = ['brak', 'umiarkowany', 'chłodny', 'zimny', 'ciepły', 'gorący']
weather_wind = ['brak', 'łagodna bryza', 'lekki wiatr', 'wiatr', 'silny wiatr']
weather_winddir = ["północy", "wschodu", "południa", "zachodu", "północnego-wschodu",
                   "północnego-zachodu", "południowego-zachodu", "południowego-wschodu"]
weather_temp = ['upalnie', 'gorąco', 'ciepło', 'chłodno', 'zimno', 'mroźnie']
weather_sky = ['bezchmurne', 'lekko zachmurzone', 'zachmurzone']
weather_shovers = ['brak', 'lekki deszcz', 'deszcz', 'deszcz ze śniegiem', 'śnieg']

class WeatherData:
    '''Data for weather.'''

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.climate = 'umiarkowany'
        self.wind = 'brak'
        self.winddir = 'północy'
        self.temperature = 'ciepło'
        self.sky = 'bezchmurne'
        self.shovers = 'brak'

        if set != None:
            if len(set.readString('climate')) > 0:
                self.climate = set.readString('climate')
                self.wind = set.readString('wind')
                self.winddir = set.readString('winddir')
                self.temperature = set.readString('temperature')
                self.sky = set.readString('sky')
                self.shovers = set.readString('shovers')

    def copyTo(self, to):
        to.climate = self.climate
        to.wind = self.wind
        to.winddir = self.winddir
        to.temperature = self.temperature
        to.sky = self.sky
        to.shovers = self.shovers

    def copy(self):
        newData = WeatherData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString('climate', self.climate)
        set.storeString('wind', self.wind)
        set.storeString('winddir', self.winddir)
        set.storeString('temperature', self.temperature)
        set.storeString('sky', self.sky)
        set.storeString('shovers', self.shovers)
        return set

################################################################################
# Weather OLC
################################################################################
__WEATHOLC_CLIMATE__ = 1
__WEATHOLC_WIND__    = 2
__WEATHOLC_WINDDIR__ = 3
__WEATHOLC_TEMP__    = 4
__WEATHOLC_SKY__     = 5
__WEATHOLC_SHOVERS__ = 6

weatholc_opt_map = {
    '1' : (__WEATHOLC_CLIMATE__, "Wybierz klimat: "),
    '2' : (__WEATHOLC_WIND__   , "Wybierz siłę wiatru: "),
    '3' : (__WEATHOLC_WINDDIR__, "Wybierz kierunek wiatru: "),
    '4' : (__WEATHOLC_TEMP__   , "Wybierz temperaturę: "),
    '5' : (__WEATHOLC_SKY__    , "Wybierz zachmurzenie: "),
    '6' : (__WEATHOLC_SHOVERS__, "Wybierz opady: ")
    }
def weatholc_menu(sock, data):
    '''Display weather settings.'''
    buf = []
    buf.append("{g1) Klimat          : {c" + data.aux('weather').climate)
    buf.append("{g2) Siła wiatru     : {c" + data.aux('weather').wind)
    buf.append("{g3) Kierunek wiatru : {c" + data.aux('weather').winddir)
    buf.append("{g4) Temperatura     : {c" + data.aux('weather').temperature)
    buf.append("{g5) Zachmurzenie    : {c" + data.aux('weather').sky)
    buf.append("{g6) Opady           : {c" + data.aux('weather').shovers)
    sock.send("\r\n".join(buf))

def weatholc_chooser(sock, data, option):
    '''figure out what field we want to edit, and display prompt.'''
    if not weatholc_opt_map.has_key(option):
        return olc.MENU_CHOICE_INVALID
    ret, mssg = weatholc_opt_map[option]

    #list our options
    opts = None
    if ret == __WEATHOLC_CLIMATE__:
        opts = weather_climate
    elif ret == __WEATHOLC_WIND__:
        opts = weather_wind
    elif ret == __WEATHOLC_WINDDIR__:
        opts = weather_winddir
    elif ret == __WEATHOLC_TEMP__:
        opts = weather_temp
    elif ret == __WEATHOLC_SKY__:
        opts = weather_sky
    elif ret == __WEATHOLC_SHOVERS__:
        opts = weather_shovers

    utils.olc_display_table(sock, opts, 4)
    sock.send_raw(mssg)
    return ret

def weatholc_parser(sock, data, choice, arg):
    '''parse out what we want to change a specified field to.'''
    try:
        arg = int(arg)
    except:
        return False
    if arg < 0:
        return False

    if choice == __WEATHOLC_CLIMATE__:
        if arg >= len(weather_climate):
            return False
        data.aux('weather').climate = weather_climate[arg]
    elif choice == __WEATHOLC_WIND__:
        if arg >= len(weather_wind):
            return False
        data.aux('weather').wind = weather_wind[arg]
    elif choice == __WEATHOLC_WINDDIR__:
        if arg >= len(weather_winddir):
            return False
        data.aux('weather').winddir = weather_winddir[arg]
    elif choice == __WEATHOLC_TEMP__:
        if arg >= len(weather_temp):
            return False
        data.aux('weather').temperature = weather_temp[arg]
    elif choice == __WEATHOLC_SKY__:
        if arg >= len(weather_sky):
            return False
        data.aux('weather').sky = weather_sky[arg]
    elif choice == _WEATHER_SHOVERS__:
        if arg >= len(weather_shovers):
            return False
        data.aux('weather').shovers = weather_shovers[arg]
    else:
        return False
    return True

def weatholc_extend_menu(sock, data):
    '''Display option in main menu.'''
    sock.send("Pogoda")

def weatholc_choice_exec(sock, data):
    '''Edit weather.'''
    olc.do_olc(sock, weatholc_menu, weatholc_chooser, weatholc_parser, None, data)
    return olc.MENU_NOCHOICE

################################################################################
# game commands
################################################################################

def cmd_weather(ch, cmd, arg):
    '''Show weather in current zone.'''
    area = zone.Zone(ch.room.locale)
    msg = "Jest " + area.aux('weather').temperature + ". Niebo jest " + area.aux('weather').sky
    if area.aux('weather').shovers != 'brak':
        msg += " i pada " + area.aux('weather').shovers
    msg += "."
    if area.aux('weather').wind != 'brak':
        msg += " Z " + area.aux('weather').winddir + " wieje " + area.aux('weather').wind + "."
    ch.send(msg)

################################################################################
# game hooks
################################################################################

def weather_hook(info):
    '''Count weather for all zones and if changed, send information about it to
    all players in this zone.'''
    #get all zones
    zones = []
    for area in os.listdir('world/zones'):
        zones.append(area)
    #count weather for each zone and show changes to players
    weather_config = {
        'umiarkowany' : {'Wiosna' : [weather_temp[2:5], weather_shovers[:4]],
                         'Lato'   : [weather_temp[1:5], weather_shovers[:3]],
                         'Jesień' : [weather_temp[2:5], weather_shovers[:4]],
                         'Zima'   : [weather_temp[3:6], weather_shovers]
                         },
        'chłodny' : {'Wiosna' : [weather_temp[3:6], weather_shovers],
                     'Lato'   : [weather_temp[2:5], weather_shovers[:3]],
                     'Jesień' : [weather_temp[3:6], weather_shovers],
                     'Zima'   : [weather_temp[3:6], [weather_shovers[0], weather_shovers[3], weather_shovers[4]]]
                     },
        'zimny' : {'Wiosna' : [weather_temp[3:6], [weather_shovers[0], weather_shovers[3], weather_shovers[4]]],
                   'Lato'   : [weather_temp[2:5], weather_shovers[:4]],
                   'Jesień' : [weather_temp[3:6], [weather_shovers[0], weather_shovers[3], weather_shovers[4]]],
                   'Zima'   : [weather_temp[4:6], weather_shovers[3:4]]
                   },
        'ciepły' : {'Wiosna' : [weather_temp[1:4], weather_shovers[:3]],
                    'Lato'   : [weather_temp[0:3], weather_shovers[:3]],
                    'Jesień' : [weather_temp[1:4], weather_shovers[:3]],
                    'Zima'   : [weather_temp[2:5], weather_shovers[:4]]
                    },
        'gorący' : {'Wiosna' : [weather_temp[1:4], weather_shovers[:2]],
                    'Lato'   : [weather_temp[0:3], weather_shovers[:1]],
                    'Jesień' : [weather_temp[1:4], weather_shovers[:2]],
                    'Zima'   : [weather_temp[1:4], weather_shovers[:3]]
                    }
        }
    season = mud.get_season()
    for z in zones:
        area = zone.Zone(z)
        if area.aux('weather').climate == 'brak':
            continue
        climate = area.aux('weather').climate
        wind = area.aux('weather').wind
        winddir = area.aux('weather').winddir
        temp = area.aux('weather').temperature
        sky = area.aux('weather').sky
        shovers = area.aux('weather').shovers
        for i in range(5):
            chance = random.randint(0, 99)
            if chance <= 20:
                chance = random.randint(0, 99)
                #change strength of wind
                if i == 0:
                    index = weather_wind.index(wind)
                    if chance < 50 and index > 0:
                        wind = weather_wind[index - 1]
                    elif chance > 49 and index < 4:
                        wind = weather_wind[index + 1]
                #change direction of wind
                elif i == 1:
                    index = weather_winddir.index(winddir)
                    if chance < 50 and index > 0:
                        winddir = weather_winddir[index - 1]
                    elif chance > 49 and index < 7:
                        winddir = weather_winddir[index + 1]
                #change temperature
                elif i == 2:
                    index = weather_config[climate][season][0].index(temp)
                    if chance < 50 and index > 0:
                        temp = weather_config[climate][season][0][index - 1]
                    elif chance > 49 and index < len(weather_config[climate][season][0]):
                        temp = weather_config[climate][season][0][index + 1]
                #change sky
                elif i == 3:
                    index = weather_sky.index(sky)
                    if chance < 50 and index > 0:
                        sky = weather_sky[index - 1]
                    elif chance > 49 and index < 2:
                        sky = weather_sky[index + 1]
                #change shovers
                elif i == 4:
                    chance = random.randint(0, len(weather_config[climate][season][1]) - 1)
                    shovers = weather_config[climate][season][1][chance]
        #if we have any chances in weather, send info about it to players in zone
        if wind != area.aux('weather').wind or winddir != area.aux('weather').winddir or temp != area.aux('weather').temperature or sky != area.aux('weather').sky or shovers != area.aux('weather').shovers:
            #create message which been send to players
            mssg = ''
            if wind != area.aux('weather').wind:
                if weather_wind.index(wind) > 0:
                    mssg += 'Zaczyna wiać ' + wind + ' z ' + winddir + '. '
                else:
                    mssg += 'Wiatr cichnie.'
                area.aux('weather').wind = wind
                area.aux('weather').winddir = winddir
            elif winddir != area.aux('weather').winddir:
                area.aux('weather').winddir = winddir
                if weather_wind.index(wind) > 0:
                    mssg += 'Wiatr zmienia kierunek i wieje z ' + winddir + '. '
            elif temp != area.aux('weather').temperature:
                if weather_temp.index(temp) < weather_temp.index(area.aux('weather').temperature):
                    mssg += 'Temperatura wzrasta. '
                else:
                    mssg += 'Temperatura spada. '
                area.aux('weather').temperature = temp
            elif sky != area.aux('weather').sky:
                area.aux('weather').sky = sky
                mssg += 'Niebo staje się ' + sky + '. '
            elif shovers != area.aux('weather').shovers:
                if weather_shovers.index(shovers) == 0:
                    mssg += 'Przestaje padać ' + area.aux('weather').shovers + '.'
                else:
                    mssg += 'Zaczyna padać ' + shovers + '.'
                area.aux('weather').shovers = shovers
            #create list of players which we must inform about weather changes
            players = []
            for ch in char.char_list():
                if ch.room.locale == area.key and not ch.room.terrain in ['Cavern', 'Inside']:
                    players.append(ch)
            #send message to players and save changes in zone
            mud.send(players, mssg)
            mudsys.do_save(area)
            
################################################################################
# initialization
################################################################################
auxiliary.install('weather', WeatherData, 'zone')

olc.extend("zedit", "W", weatholc_extend_menu, weatholc_choice_exec, None, None, None)

mudsys.add_cmd("pogoda", None, cmd_weather, "player", True)

hooks.add('weather', weather_hook)

def chk_can_weather(ch, cmd):
    #if player is inside, he cannot check weather
    if ch.room.terrain in ['Cavern', 'Inside']:
        ch.send("Nie widzisz nieba.")
        return False
    if not ch.pos in ["sitting", "standing", "flying"]:
        if ch.pos == "sleeping":
            pozycja = "śpisz"
        if ch.pos == "unconscious":
            pozycja = "jesteś nieprzytomny"
        ch.send("Nie możesz sprawdzić jaka jest pogoda, kiedy " + pozycja + ".")
        return False

mudsys.add_cmd_check("pogoda", chk_can_weather)

def __unload__():
    '''removes the hooks for weather.'''
    hooks.remove('weather', weather_hook)
