#ifndef PYZONE_H
#define PYZONE_H
/*
 * Copyright (C), 2010 Bartek thindil Jasicki
 *
 * This file is part of Anaria.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
//*****************************************************************************
//
// pyzone.h
//
// Contains a python zone module, and an Zone class that is a python 
// wrapper for Anaria zones. If you wish to give python access to more
// features of an zone, it should NOT be done by editing pyzone.c! Use 
// PyZone_addGetSetter and PyZone_addMethod in a new module implementing 
// the feature you want to give Python access to.
//
//*****************************************************************************

// used by pymudsys. Do not touch!
PyTypeObject PyZone_Type;

// initialize zones for use. This must be called AFTER all other modules
// have added in new get/setters and methods to pyroom
PyMODINIT_FUNC     init_PyZone(void);
PyObject            *newPyZone(ZONE_DATA *zone);
ZONE_DATA *PyZone_AsZone(PyObject *zone);

//
// checks to see if the PyObject is a PyZone
int          PyZone_Check(PyObject *value);

//
// getters allow Python to access pieces of the Zone module. Setters allow
// Python to change pieces of the zone module. Getters are called when Python
// tries to get the value of some variable on the object, and setters are called
// when Python tries to set the value of some variable on the object. Get and
// Set do not both need to be supplied. Examples of how to add new getters and
// setters is presented in pyzone.c
void PyZone_addGetSetter(const char *name, void *g, void *s, const char *doc);

//
// Adds a new method function (i.e. void *f) to the Zone class. Name is the 
// name of the function, f is the PyCFunction implementing the new method, 
// flags is the type of method beings used (almost always METH_VARARGS), and
// dog is an (optional) description of what the method does. For examples on
// how to add new methods, see pyzone.c
void PyZone_addMethod(const char *name, void *f, int flags, const char *doc);

#endif
